package com.btcpiyush.ads.google_applovin_unity_ads

import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.text.TextUtils
import android.util.Log
import androidx.annotation.NonNull
import com.applovin.mediation.*
import com.applovin.mediation.ads.MaxInterstitialAd
import com.applovin.mediation.ads.MaxRewardedAd
import com.applovin.sdk.AppLovinMediationProvider
import com.applovin.sdk.AppLovinSdk
import com.applovin.sdk.AppLovinSdkSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppLovinSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.GoogleAdsSetting
import com.btcpiyush.ads.google_applovin_unity_ads.banner_ad.BannerAdControllerManager
import com.btcpiyush.ads.google_applovin_unity_ads.banner_ad.BannerAdViewFactory
import com.btcpiyush.ads.google_applovin_unity_ads.interestial_ad.AdmobInterstitial
import com.btcpiyush.ads.google_applovin_unity_ads.native_ad.NativeAdControllerManager
import com.btcpiyush.ads.google_applovin_unity_ads.native_ad.NativeAdViewFactory
import com.facebook.ads.*
import com.google.android.gms.ads.*
import com.google.android.gms.ads.AdError
import com.google.android.gms.ads.AdListener
import com.google.android.gms.ads.appopen.AppOpenAd
import com.google.android.gms.ads.interstitial.InterstitialAd
import com.google.android.gms.ads.interstitial.InterstitialAdLoadCallback
import com.google.android.gms.ads.rewarded.RewardItem
import com.google.android.gms.ads.rewarded.RewardedAd
import com.google.android.gms.ads.rewarded.RewardedAdLoadCallback
import com.google.gson.Gson
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.platform.PlatformViewRegistry
import java.util.*
import com.facebook.ads.AdError as FBAdError
import com.facebook.ads.InterstitialAd as FBInterstitialAd

class GoogleApplovinUnityAdsPlugin : FlutterPlugin, ActivityAware, MethodCallHandler {

    companion object {
        val handler = Handler(Looper.getMainLooper())
        const val TAG = "GoogleApplovinPlugin"
        const val CHANNEL_NAME = "dev.bitcoding.ads/google_applovin_unity_ads"
        const val CHANNEL_BANNER_NAME = "dev.bitcoding.ads/banner"
        const val CHANNEL_INTERESTIAL_NAME = "dev.bitcoding.ads/interstitial"
        const val CHANNEL_REWARD_NAME = "dev.bitcoding.ads/reward"
        const val CHANNEL_NATIVE_NAME = "dev.bitcoding.ads/native"
        const val METHOD_ADS_API_SETTING = "call.ads.api.setting"
        const val METHOD_LOAD_ADS = "call.ads.interestial"
        const val METHOD_REWARD_LOAD_ADS = "call.ads.rewarded"
        const val METHOD_OPEN_ADS = "call.ads.openad"
        const val METHOD_LOAD_BANNER_ADS = "call.ads.banner"
        const val METHOD_LOAD_NATIVE_ADS = "call.ads.native"
        const val METHOD_SUCCESS = "SUCCESS"
        const val METHOD_ERROR = "ERROR"
        const val METHOD_DISPOSE_BANNER_ADS = "dispose.ads.banner"
        const val METHOD_DISPOSE_NATIVE_ADS = "dispose.ads.native"
        const val METHOD_BANNER_ADS = "ads.banner"
        const val METHOD_NATIVE_ADS = "ads.native"
        var interestialAdInterval: Long = 0L
        var appLovinSDK: AppLovinSdk? = null
    }

    private var applicationContext: Context? = null

    private var methodChannel: MethodChannel? = null
    private var interstitialChannel: MethodChannel? = null
    private var rewardChannel: MethodChannel? = null
    private lateinit var messenger: BinaryMessenger
    private var activity: Activity? = null
    private var mGoogleInterstitialAd: InterstitialAd? = null
    private var mFacebookInterstitialAd: com.facebook.ads.InterstitialAd? = null
    private var mGoogleRewardedAd: RewardedAd? = null
    private var mFacebookRewardedAd: RewardedVideoAd? = null
    private var mOpenAd: AppOpenAd? = null


    //    private var geAdMobInterstitialAd: AHAdMobInterstitialAd? = null
    private var ahWrapperListener: InterstitialAdLoadCallback? = null
    private var mInterstitialAdLoadCallback: InterstitialAdLoadCallback? = null
    private var mOpenAdLoadCallback: AppOpenAd.AppOpenAdLoadCallback? = null
    private var mOpenAdShowCallback: FullScreenContentCallback? = null
    private var interstitialAdListener: InterstitialAdListener? = null
    private var mInterestialAdShowCallback: FullScreenContentCallback? = null
    private var mRewardedAdShowCallback: FullScreenContentCallback? = null
    private var mFacebookRewardedAdLoadCallback: RewardedVideoAdListener? = null

    //    private var geAdMobRewardAd: AHAdMobRewardedAd? = null
    private var ahRewardWrapperListener: RewardedAdLoadCallback? = null
    private var mRewardedAdLoadCallback: RewardedAdLoadCallback? = null

    /*    private var mUnityAdsLoadCallback: IUnityAdsLoadListener? = null
        private var mUnityAdsShowCallback: IUnityAdsShowListener? = null*/
    private var mInterstitialAdIsLoading: Boolean = false
    private var mRewardAdIsLoading: Boolean = false
    private var mOpenAdIsLoading: Boolean = false
    private var mOpenAdIsShowing: Boolean = false
    private var progressBar: ProgressDialog? = null
    private var mInterstitialAppLovinAd: MaxInterstitialAd? = null
    private var mRewardedAppLovinAd: MaxRewardedAd? = null
    private var mInterstitialAppLovinCallback: MaxAdListener? = null
    private var mRewardedAppLovinCallback: MaxRewardedAdListener? = null
    private lateinit var platformViewRegistry: PlatformViewRegistry
    private var settingResponse: AppSettings? = null
    private var isRewardVideoShowCompleted = false
    private lateinit var pluginBinding: FlutterPlugin.FlutterPluginBinding

    override fun onAttachedToEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        pluginBinding = binding
        applicationContext = binding.applicationContext
        platformViewRegistry = binding.platformViewRegistry
        messenger = binding.binaryMessenger
        methodChannel = MethodChannel(messenger, CHANNEL_NAME)
        interstitialChannel = MethodChannel(binding.binaryMessenger, CHANNEL_INTERESTIAL_NAME)
//        rewardChannel = MethodChannel(messenger, CHANNEL_REWARD_NAME)
//        rewardChannel.setMethodCallHandler(this)
        methodChannel?.setMethodCallHandler(this)
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            METHOD_ADS_API_SETTING -> {
                loadSettingAPI(result, call.argument<String>("json_data").toString())
            }

            METHOD_LOAD_ADS -> {
                activity?.let {
                    settingResponse?.let {
                        it.adSetting?.interestialAdsIntervalCount?.let { count ->
                            it.adSetting?.interestialAdsIntervalCount = count + 1
                        }
                    }
                    loadInterestialAds(it, result)
                }
            }

            METHOD_REWARD_LOAD_ADS -> {
                activity?.let {
                    loadRewardAds(it, result)
                }
            }

            METHOD_OPEN_ADS -> {
                activity?.let {
                    requestGoogleOpenAd(it, result)
                }
            }

            METHOD_LOAD_BANNER_ADS -> {
                val id = call.argument<String>("id")
                activity?.let {
                    loadBannerAd(it, result, id)
                }
            }

            METHOD_LOAD_NATIVE_ADS -> {
                val id = call.argument<String>("id")
                activity?.let {
                    loadNativeAd(it, result, id)
                }
            }

            METHOD_DISPOSE_BANNER_ADS -> {
                val id = call.argument<String>("id")
                activity?.let {
                    destroyBannerAd(it, result, id)
                }
            }

            METHOD_DISPOSE_NATIVE_ADS -> {
                val id = call.argument<String>("id")
                activity?.let {
                    destroyNativeAd(it, result, id)
                }
            }

            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        methodChannel?.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {}

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        activity = binding.activity
    }

    override fun onDetachedFromActivity() {}

    private fun loadSettingAPI(result: MethodChannel.Result?, json_data: String) {
        try {
            // sendSuccess(result, "SUCCESS")
            settingResponse = Gson().fromJson(json_data, AppSettings::class.java)
            settingResponse?.let {
                activity?.let { act ->
                    progressBar = ProgressDialog(
                        act,
                        it.adSetting?.adsLoadingText ?: "Ads will loading..."
                    )
                    if ((it.adSetting?.appVersionCode ?: 1).toLong() == getVersionCode()) {
                        it.adsSequence.clear()
                    }
                    loadSdk(act, result)
                }
            }

        } catch (exp: Exception) {
        }
    }

    private fun getVersionCode() : Long {
        activity?.let { context ->
            try {
                val pInfo = context.packageManager.getPackageInfo(context.packageName, 0)
                return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                    pInfo.longVersionCode
                } else {
                    pInfo.versionCode.toLong()
                }
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace()
            }
        }

        return 1L
    }


    private fun findMatch(s: String, strings: ArrayList<String>): Boolean {
        return strings.any { s.contains(it) }
    }

    private fun loadSdk(act: Activity, result: MethodChannel.Result? = null) {
        settingResponse?.let {
            interstitialChannel?.setMethodCallHandler(AdmobInterstitial(pluginBinding, act, it))
            if (it.adsSequence.size > 0) {
                var isBannerAds: Boolean = it.customAds?.isBannerAds == true
                var isNativeAds: Boolean = it.customAds?.isNativeAds == true

                if (findMatch(act.getString(R.string.google_ads).lowercase(), it.adsSequence)) {
                    it.googleAds?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isOpenAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds
                            MobileAds.initialize(act) {}

                        }
                    }
                }

                if (findMatch(act.getString(R.string.app_lovin).lowercase(), it.adsSequence)) {
                    it.appLovin?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds
                            _applovinSDKInitialize(act, it)
                        }
                    }
                }

                if (findMatch(act.getString(R.string.facebook_ads).lowercase(), it.adsSequence)) {
                    it.facebookAds?.let {
                        if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                            isBannerAds = it.isBannerAds == true || isBannerAds
                            isNativeAds = it.isNativeAds == true || isNativeAds

                            if (BuildConfig.DEBUG) {
                                AdSettings.turnOnSDKDebugger(act)
                                AdSettings.isTestMode(act)
                                AdSettings.addTestDevice("55ebd8be-199d-4bdc-85eb-69e6e7fa6d66")
                            }

                            AudienceNetworkAds.initialize(act)
                        }
                    }
                }

                if (isBannerAds) {
                    platformViewRegistry.registerViewFactory(
                        METHOD_BANNER_ADS,
                        BannerAdViewFactory(act, it)
                    )
                }

                if (isNativeAds) {
                    platformViewRegistry?.registerViewFactory(
                        METHOD_NATIVE_ADS,
                        NativeAdViewFactory(act, it)
                    )
                }
            }
            sendSuccess(result, METHOD_SUCCESS)
        } ?: kotlin.run {
            sendSuccess(result, METHOD_ERROR)
        }
    }

    private fun loadSdk(act: Activity, result: MethodChannel.Result? = null, pos: Int = 0) {
        settingResponse?.let {
            interstitialChannel?.setMethodCallHandler(AdmobInterstitial(pluginBinding, act, it))
            if (it.adsSequence.size > 0) {
                if (it.adsSequence.size > pos) {
                    Log.e(TAG, "loadSdk: $pos   ${it.adsSequence[pos].lowercase()}")
                    when (it.adsSequence[pos].lowercase()) {
                        act.getString(R.string.google_ads).lowercase() -> {
                            initMobSdk(act, result, pos)
                        }
                        act.getString(R.string.facebook_ads).lowercase() -> {
                            initFacebookSdk(act, result, pos)
                        }
                        act.getString(R.string.app_lovin).lowercase() -> {
                            initAppLovinSDK(act, result, pos)
                        }
                        /*getString(R.string.unity_ads).lowercase() -> {
                             initUnitySDK(result, pos)
                         }*/
                        else -> {
                            loadSdk(act, result, pos + 1)
                        }
                    }
                } else {
                    Log.e(TAG, "loadSdk: $pos   Other Pos Data}")
                    val handler = Handler(Looper.getMainLooper())
                    handler.postDelayed({
                        mInterstitialAdIsLoading = false
                        mRewardAdIsLoading = false

                        progressBar?.dismiss()

                        if (it.googleAds?.isBannerAds == true ||
                            it.appLovin?.isBannerAds == true ||
                            it.facebookAds?.isBannerAds == true ||
                            it.customAds?.isBannerAds == true
                        ) {
                            platformViewRegistry.registerViewFactory(
                                METHOD_BANNER_ADS,
                                BannerAdViewFactory(act, it)
                            )
                        }

                        if (it.googleAds?.isNativeAds == true ||
                            it.appLovin?.isNativeAds == true ||
                            it.facebookAds?.isNativeAds == true ||
                            it.customAds?.isNativeAds == true
                        ) {
                            platformViewRegistry?.registerViewFactory(
                                METHOD_NATIVE_ADS,
                                NativeAdViewFactory(act, it)
                            )
                        }

                        sendSuccess(result, "SUCCESS")

                        // requestGoogleOpenAd(act)

                    }, 10)
                }
            } else {
                Log.e(TAG, "loadSdk: $pos   Other Pos Data}")
                sendSuccess(result, "SUCCESS")
            }
        }
    }

    private fun initMobSdk(act: Activity, result: MethodChannel.Result?, pos: Int) {
        try {
            settingResponse?.let { setting ->
                setting.googleAds?.let {
                    if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isOpenAds == true || it.isRewardedAds == true) {
                        MobileAds.initialize(act) {
                            loadSdk(act, result, pos + 1)
                        }
                        return@initMobSdk
                    }
                }
            }
        } catch (e: Exception) {
        }

        loadSdk(act, result, pos + 1)
    }

    private fun initFacebookSdk(act: Activity, result: MethodChannel.Result?, pos: Int) {
        try {
            settingResponse?.let {
                it.facebookAds?.let {
                    if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
                        if (BuildConfig.DEBUG) {
                            AdSettings.turnOnSDKDebugger(act)
                            AdSettings.isTestMode(act)
                            AdSettings.addTestDevice("55ebd8be-199d-4bdc-85eb-69e6e7fa6d66")
                        }
                        AudienceNetworkAds.initialize(act)
                    }
                }
            }
        } catch (e: Exception) {
        }
        loadSdk(act, result, pos + 1)
    }

    private fun initAppLovinSDK(act: Activity, result: MethodChannel.Result?, pos: Int) {
        settingResponse?.let {
            it.appLovin?.let {
                if (it.isBannerAds == true || it.isInterstitialAds == true || it.isNativeAds == true || it.isRewardedAds == true) {
//                    AppLovinSdk.getInstance(act).mediationProvider = "max"
//                    AppLovinSdk.initializeSdk(act)
                    _applovinSDKInitialize(act, it)
                }
            }
        }
        loadSdk(act, result, pos + 1)
    }

    private fun _applovinSDKInitialize(act: Activity, it: AppLovinSetting) {
        var sdkKeyToUse = it.appLovinSdkKey
        Log.e(TAG, "initAppLovinSDK: $sdkKeyToUse")
        if (TextUtils.isEmpty(sdkKeyToUse)) {
            try {
                applicationContext?.let {
                    val packageManager = it.packageManager
                    val packageName = it.packageName
                    val applicationInfo = packageManager.getApplicationInfo(
                        packageName,
                        PackageManager.GET_META_DATA
                    )
                    val metaData = applicationInfo.metaData
                    sdkKeyToUse = metaData.getString("applovin.sdk.key", "")
                    Log.e(TAG, "$sdkKeyToUse")
                }
            } catch (th: Throwable) {
                Log.e(TAG, "Unable to retrieve SDK key from Android Manifest: $th")
            }
            // check(!TextUtils.isEmpty(sdkKeyToUse)) { "Unable to initialize AppLovin SDK - no SDK key provided and not found in Android Manifest!" }
        }

        Log.e(TAG, "$sdkKeyToUse")

        // Initialize SDK
        appLovinSDK = AppLovinSdk.getInstance(
            sdkKeyToUse,
            AppLovinSdkSettings(act),
            act
        )

        appLovinSDK?.mediationProvider = AppLovinMediationProvider.MAX
        appLovinSDK?.settings?.setVerboseLogging(true)

        appLovinSDK?.initializeSdk { configuration ->
            Log.e(TAG, "APPLOVIN SDK initialized")
        }
    }

    /* private fun initUnitySDK(result: MethodChannel.Result?, pos: Int) {
         settingResponse?.let {
             it.unityAds?.let {
                 if (it.isBannerAds == true || it.isInterstitialAds == true || it.isRewardAds == true) {
                     if (it.unityId.isNullOrEmpty()) {
                         loadSdk(result, pos + 1)
                         return
                     }
                     UnityAds.initialize(
                         this,
                         it.unityId,
                         false,
                         object : IUnityAdsInitializationListener {

                             override fun onInitializationComplete() {
                                 showLog("UNITY SUCCESS")
                                 sendSuccess(result, "SUCCESS")
                                 loadSdk(result, pos + 1)
                             }

                             override fun onInitializationFailed(
                                 error: UnityAds.UnityAdsInitializationError?,
                                 message: String?
                             ) {
                                 showLog("UNITY ERROR -- ${message.toString()}")
                                 sendError(result, error?.ordinal.toString(), message.toString())
                                 loadSdk(result, pos + 1)
                             }
                         })
                 }
             } ?: kotlin.run {
                 loadSdk(result, pos + 1)
             }
         } ?: kotlin.run {
             loadSdk(result, pos + 1)
         }
     }
 */

    private fun loadInterestialAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int = 0
    ) {
        settingResponse?.let {
            val isInterestial = ((it.adSetting?.interestialAdsInterval
                ?: 0) > 0 && it.adSetting?.interestialAdsIntervalCount?.mod(
                it.adSetting?.interestialAdsInterval ?: 0
            ) == 0 && (interestialAdInterval + ((it.adSetting?.interestialAdsTimingInterval
                ?: 0) * 1000) <= Calendar.getInstance().timeInMillis || interestialAdInterval == 0L))

            if (it.adsSequence.size > 0 && it.adsSequence.size > pos) {
                when (it.adsSequence[pos].lowercase()) {
                    act.getString(R.string.google_ads).lowercase() -> {

                        it.googleAds?.let { google_ads ->
                            if (isInterestial || (google_ads.interstitialInterval > 0 && it.adSetting?.interestialAdsIntervalCount?.mod(
                                    google_ads.interstitialInterval
                                ) == 0)
                            ) {
                                loadGoogleAdsInterestialAds(
                                    act, result, pos,
                                    google_ads
                                )
                            } else {
                                loadInterestialAds(act, result, pos + 1)
                            }
                        } ?: kotlin.run {
                            loadInterestialAds(act, result, pos + 1)
                        }
                    }

                    act.getString(R.string.facebook_ads).lowercase() -> {
                        it.facebookAds?.let { facebook_ads ->
                            if (isInterestial || (facebook_ads.interstitialInterval > 0 && it.adSetting?.interestialAdsIntervalCount?.mod(
                                    facebook_ads.interstitialInterval
                                ) == 0)
                            ) {
                                loadFacebookAdsInterestialAds(act, result, pos)
                            } else {
                                loadInterestialAds(act, result, pos + 1)
                            }
                        } ?: kotlin.run {
                            loadInterestialAds(act, result, pos + 1)
                        }
                    }

                    act.getString(R.string.app_lovin).lowercase() -> {
                        if ((it.appLovin?.isUnlimitedAds == true) || isInterestial) {
                            loadAppLovinInterestialAd(act, result, pos)
                        } else {
                            loadInterestialAds(act, result, pos + 1)
                        }
                    }

                    act.getString(R.string.custom_ads).lowercase() -> {
                        if (isInterestial) {
                            loadCustomInterestialAds(act, result, pos)
                        } else {
                            loadInterestialAds(act, result, pos + 1)
                        }
                    }

                    else -> {
                        loadInterestialAds(act, result, pos + 1)
                    }
                }
            } else {
                methodChannel?.invokeMethod(METHOD_ERROR, null)
                mInterstitialAdIsLoading = false
                progressBar?.dismiss()
            }
        }
    }

    private fun loadExtraGoogleAdsInterestialAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int,
        extraGooglepos: Int,
        it: ArrayList<GoogleAdsSetting?>
    ) {
        if (it.size > 0 && it.size > extraGooglepos) {
            it[extraGooglepos]?.let { google_ads ->
                loadGoogleAdsInterestialAds(act, result, pos, google_ads, extraGooglepos, it)
            } ?: kotlin.run {
                loadExtraGoogleAdsInterestialAds(
                    act, result, pos, extraGooglepos + 1,
                    it
                )
            }
        } else {
            loadInterestialAds(act, result, pos + 1)
        }
    }

    private fun loadGoogleAdsInterestialAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int,
        it: GoogleAdsSetting,
        extraGooglepos: Int = -1,
        otherList: ArrayList<GoogleAdsSetting?> = arrayListOf()
    ) {
        if (it.isInterstitialAds == true) {
            if (it.interstitialId.isNullOrEmpty()) {
                if (otherList.size > 0 && otherList.size > extraGooglepos) {
                    loadExtraGoogleAdsInterestialAds(
                        act, result, pos, extraGooglepos + 1,
                        otherList
                    )
                } else {
                    loadInterestialAds(act, result, pos + 1)
                }
                return@loadGoogleAdsInterestialAds
            }

            if (mInterstitialAdIsLoading) {
                showLog("Interestial Ad is Already Loading Progress...")
                methodChannel?.invokeMethod(METHOD_ERROR, null)
                return@loadGoogleAdsInterestialAds
            }

            if (mGoogleInterstitialAd != null) {
                showLog("Interestial Ad is Already Loading Completed")
                showInterestialAds(act, result, pos)
                return@loadGoogleAdsInterestialAds
            }

            mInterstitialAdIsLoading = true
            progressBar?.show()

            if (mInterestialAdShowCallback == null) {
                mInterestialAdShowCallback = object : FullScreenContentCallback() {
                    override fun onAdClicked() {
                        showLog("   --------  Interestial onAdClicked ---   ")
                        // sendSuccess(result, "   --------  Interestial onAdClicked ---   ")
                        super.onAdClicked()
                    }

                    override fun onAdDismissedFullScreenContent() {
                        super.onAdDismissedFullScreenContent()
                        mGoogleInterstitialAd = null
//                                geAdMobInterstitialAd = null
                        progressBar?.dismiss()
                        interestialAdInterval = Calendar.getInstance().timeInMillis
                        showLog("   -------- onAdDismissedFullScreenContent Show  $mGoogleInterstitialAd")
                        methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                    }

                    override fun onAdShowedFullScreenContent() {
                        super.onAdShowedFullScreenContent()

                        progressBar?.dismiss()
                        showLog(" -------- onAdShowedFullScreenContent Show  --- ")
                        // sendSuccess(result, "   -------- onAdShowedFullScreenContent Show  ---   ")
                    }

                    override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                        super.onAdFailedToShowFullScreenContent(adError)
                        showLog(
                            " -------- onAdFailedToShowFullScreenContent Show  --- $adError"
                        )
                        mGoogleInterstitialAd = null
//                                geAdMobInterstitialAd = null
                        handler.postDelayed({
                            if (otherList.size > 0 && otherList.size > extraGooglepos) {
                                loadExtraGoogleAdsInterestialAds(
                                    act, result, pos, extraGooglepos + 1,
                                    otherList
                                )
                            } else {
                                loadInterestialAds(act, result, pos + 1)
                            }
                        }, 10L)
                        /*result?.error(
                            adError.code.toString(),
                            "   -------- onAdShowedFullScreenContent Show  ---   ${adError.message}",
                            null
                        )*/
                    }

                    override fun onAdImpression() {
                        super.onAdImpression()
                        showLog(
                            " -------- onAdImpression Show  --- "
                        )
                        /*sendSuccess(
                            result,
                            "   -------- onAdImpression Show  ---   "
                        )*/
                    }
                }
            }

            if (mInterstitialAdLoadCallback == null) {
                mInterstitialAdLoadCallback =
                    object : InterstitialAdLoadCallback() {
                        override fun onAdLoaded(interstitialAd: InterstitialAd) {
                            super.onAdLoaded(interstitialAd)
                            mInterstitialAdIsLoading = false
                            mGoogleInterstitialAd = interstitialAd
                            handler.postDelayed({
                                result?.let {
                                    progressBar?.dismiss()
                                    showInterestialAds(act, result, pos)
                                }
                            }, 10L)

                            mGoogleInterstitialAd?.fullScreenContentCallback =
                                mInterestialAdShowCallback
                        }

                        override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                            super.onAdFailedToLoad(loadAdError)
                            mInterstitialAdIsLoading = false
                            handler.postDelayed({
                                if (otherList.size > 0 && otherList.size > extraGooglepos) {
                                    loadExtraGoogleAdsInterestialAds(
                                        act, result, pos, extraGooglepos + 1,
                                        otherList
                                    )
                                } else {
                                    loadInterestialAds(act, result, pos + 1)
                                }
                            }, 10L)

                            /*handler.postDelayed({
                                showCustomAds()
                            }, 10L)*/

                        }
                    }
            }

            /*      setting.adSetting?.let {
                      Log.e(TAG, "loadGoogleAdsInterestialAds: ADNW_DEBUG_LOGGER " )
                      if (it.isGeoSDKFlag && (it.geoSDKKey ?: "").isNotEmpty()) {
                          Log.e(TAG, "loadGoogleAdsInterestialAds: isGeoSDKFlag is Set And  isGeoSDKFlag is Key is Set  ADNW_DEBUG_LOGGER" )
                          if (geAdMobInterstitialAd == null) {
                              geAdMobInterstitialAd = AHAdMobInterstitialAd()
                          }
                      }
                  }*/

            mInterstitialAdLoadCallback?.let { callback ->
                Log.e(TAG, "loadGoogleAdsInterestialAds: ADNW_DEBUG_LOGGER callback is Set ")
                /*geAdMobInterstitialAd?.let {
                    Log.e(TAG, "loadGoogleAdsInterestialAds:ADNW_DEBUG_LOGGER  geAdMobInterstitialAd is Set  " )
                    ahWrapperListener = AppHarbr.addInterstitial(
                        AdSdk.ADMOB,
                        it,
                        callback
                    ) { _: Any?, _: String?, _: AdFormat?, _: Array<AdBlockReason?>? ->
                        Log.e(
                            "$TAG",
                            "AppHarbr - ADNW_DEBUG_LOGGER  On addInterstitial callback of ahWrapperListener"
                        )
                    }
                }*/

                ahWrapperListener?.let { callback ->
                    Log.e(
                        TAG,
                        "loadGoogleAdsInterestialAds: ADNW_DEBUG_LOGGER  ahWrapperListener is Set"
                    )
                    InterstitialAd.load(
                        act, it.interstitialId!!, getGoogleAdRequest(),
                        callback
                    )
                } ?: kotlin.run {
                    Log.e(
                        TAG,
                        "loadGoogleAdsInterestialAds: ADNW_DEBUG_LOGGER  ahWrapperListener isn't Set"
                    )
                    InterstitialAd.load(
                        act, it.interstitialId!!, getGoogleAdRequest(),
                        callback
                    )
                }
            }

        } else {
            if (otherList.size > 0 && otherList.size > extraGooglepos) {
                loadExtraGoogleAdsInterestialAds(
                    act, result, pos, extraGooglepos + 1,
                    otherList
                )
            } else {
                loadInterestialAds(act, result, pos + 1)
            }
        }
    }

    private fun loadFacebookAdsInterestialAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        settingResponse?.let {
            it.facebookAds?.let {
                if (it.isInterstitialAds == true) {
                    if (it.interstitialId?.isNullOrEmpty() == true) {
                        loadInterestialAds(act, result, pos + 1)
                        return@loadFacebookAdsInterestialAds
                    }

                    if (mInterstitialAdIsLoading) {
                        showLog("Interestial Ad is Already Loading Progress...")
                        methodChannel?.invokeMethod(METHOD_ERROR, null)
                        return
                    }

                    if (mFacebookInterstitialAd != null) {
                        showLog("Interestial Ad is Already Loading Completed")
                        methodChannel?.invokeMethod(METHOD_ERROR, null)
                        return
                    }

                    mInterstitialAdIsLoading = true

                    progressBar?.show()

                    if (interstitialAdListener == null) {
                        interstitialAdListener = object : InterstitialAdListener {
                            override fun onError(p0: Ad?, p1: com.facebook.ads.AdError?) {
                                showLog(
                                    " -------- onAdFailedToShowFullScreenContent Show  --- $p1"
                                )
                                mFacebookInterstitialAd = null
                                mInterstitialAdIsLoading = false
                                handler.postDelayed({
                                    loadInterestialAds(act, result, pos + 1)
                                }, 10L)
                            }

                            override fun onAdLoaded(p0: Ad?) {
                                mInterstitialAdIsLoading = false
                                handler.postDelayed({
                                    result?.let {
                                        progressBar?.dismiss()
                                        showFacebookInterestialAds(act, result, pos)
                                    }
                                }, 10L)
                            }

                            override fun onAdClicked(p0: Ad?) {}

                            override fun onLoggingImpression(p0: Ad?) {}

                            override fun onInterstitialDisplayed(p0: Ad?) {
                                progressBar?.dismiss()
                                interestialAdInterval = Calendar.getInstance().timeInMillis
                            }

                            override fun onInterstitialDismissed(p0: Ad?) {
                                mFacebookInterstitialAd = null
                                mInterstitialAdIsLoading = false
                                progressBar?.dismiss()
                                interestialAdInterval = Calendar.getInstance().timeInMillis
                                showLog("   -------- onAdDismissedFullScreenContent Show  $mGoogleInterstitialAd")
                                methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                            }
                        }
                    }

                    interstitialAdListener?.let { adsListener ->
                        mFacebookInterstitialAd =
                            FBInterstitialAd(act, it.interstitialId)
                        mFacebookInterstitialAd?.loadAd(
                            mFacebookInterstitialAd?.buildLoadAdConfig()
                                ?.withAdListener(interstitialAdListener)?.build()
                        )
                    }

                } else {
                    loadInterestialAds(act, result, pos + 1)
                }
            } ?: kotlin.run {
                loadInterestialAds(act, result, pos + 1)
            }
        }
    }

    private fun loadAppLovinInterestialAd(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        appLovinSDK?.let { appLovinSDK ->
            settingResponse?.let {
                it.appLovin?.let {
                    if (it.isInterstitialAds == true) {

                        if (it.appLovinInterestialID.isNullOrEmpty()) {
                            loadInterestialAds(act, result, pos + 1)
                            return@loadAppLovinInterestialAd
                        }

                        if (mInterstitialAdIsLoading) {
                            progressBar?.dismiss()
                            showLog("Interestial Ad is Already Loading Progress...")
                            methodChannel?.invokeMethod(METHOD_ERROR, null)
                            return@loadAppLovinInterestialAd
                        }

                        if (mInterstitialAppLovinAd != null) {
                            progressBar?.dismiss()
                            showLog("Interestial Ad is Already Loading Completed")
                            methodChannel?.invokeMethod(METHOD_ERROR, null)
                            return@loadAppLovinInterestialAd
                        }

                        mInterstitialAdIsLoading = true
                        showLog("Interestial Ad AppLovin is Started ...")

                        progressBar?.show()
                        if (mInterstitialAppLovinCallback == null) {
                            mInterstitialAppLovinCallback = object : MaxAdListener {

                                override fun onAdLoaded(maxAd: MaxAd?) {
                                    showLog(
                                        " -------- onAdLoaded apploving 12312  --- "
                                    )
                                    mInterstitialAdIsLoading = false
                                    handler.postDelayed({
                                        result?.let {

                                            progressBar?.dismiss()
                                            showAppLovinInterestialAds(act, result, pos)
                                        }
                                    }, 10L)
                                }

                                override fun onAdLoadFailed(adUnitId: String?, error: MaxError?) {
                                    showLog("onAdLoadFailed: AppLovin Ad Failed  ------ ")
                                    mInterstitialAppLovinAd = null
                                    mInterstitialAdIsLoading = false

                                    progressBar?.dismiss()
                                    handler.postDelayed({
                                        loadInterestialAds(act, result, pos + 1)
                                    }, 10L)
                                }

                                override fun onAdDisplayFailed(maxAd: MaxAd?, error: MaxError?) {
                                    // Interstitial ad failed to display. AppLovin recommends that you load the next ad.
                                    // mInterstitialAppLovinAd?.loadAd()
                                    mInterstitialAppLovinAd = null
                                    mInterstitialAdIsLoading = false
                                    showLog("onAdDisplayFailed: AppLovin Ad Failed   ------    ")
                                    handler.postDelayed({
                                        loadInterestialAds(act, result, pos + 1)
                                    }, 10L)
                                }

                                override fun onAdDisplayed(maxAd: MaxAd?) {
                                    showLog(
                                        " -------- onAdDisplayed apploving 12312  --- "
                                    )
                                }

                                override fun onAdClicked(maxAd: MaxAd?) {

                                }

                                override fun onAdHidden(maxAd: MaxAd?) {
                                    // Interstitial ad is hidden. Pre-load the next ad
                                    mInterstitialAdIsLoading = false
                                    mInterstitialAppLovinAd = null
                                    showLog(
                                        " -------- onAdHidden apploving 12312  --- "
                                    )
                                    interestialAdInterval = Calendar.getInstance().timeInMillis
                                    methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                                }

                            }
                        }

                        mInterstitialAppLovinAd = MaxInterstitialAd(
                            it.appLovinInterestialID, appLovinSDK, act
                        )
                        mInterstitialAppLovinAd?.setListener(mInterstitialAppLovinCallback)

                        // Load the first ad
                        mInterstitialAppLovinAd?.loadAd()
                    } else {
                        loadInterestialAds(act, result, pos + 1)
                    }
                } ?: kotlin.run {
                    loadInterestialAds(act, result, pos + 1)
                }
            }
        } ?: kotlin.run {
            loadInterestialAds(act, result, pos + 1)
        }

    }

    /* private fun loadUnityAds(result: MethodChannel.Result? = null, unityAdId: String, pos: Int) {
         settingResponse?.let {
             it.unityAds?.let {
                 val isAds = when (unityAdId) {
                     UNITY_INTERESTIAL_AD_ID -> {
                         it.isInterstitialAds == true
                     }
                     UNITY_REWARD_AD_ID -> {
                         it.isRewardAds == true
                     }
                     else -> {
                         false
                     }
                 }

                 if (isAds) {

                     when (unityAdId) {
                         UNITY_INTERESTIAL_AD_ID -> {
                             if (mInterstitialAdIsLoading) {

                                 progressBar?.dismiss()
                                 showLog("Unity Interestial Ad is Already Loading Progress...")
                                 channel?.invokeMethod(METHOD_ERROR, null)
                                 return
                             }
                         }
                         UNITY_REWARD_AD_ID -> {
                             if (mRewardAdIsLoading) {

                                 progressBar?.dismiss()
                                 showLog("Unity Reward Ad is Already Loading Progress...")
                                 channel?.invokeMethod(METHOD_ERROR, null)
                                 return
                             }
                         }
                     }


                     progressBar?.show()

                     when (unityAdId) {
                         UNITY_INTERESTIAL_AD_ID -> {
                             mInterstitialAdIsLoading = true
                         }
                         UNITY_REWARD_AD_ID -> {
                             mRewardAdIsLoading = true
                         }
                     }

                     if (mUnityAdsShowCallback == null) {
                         mUnityAdsShowCallback = object : IUnityAdsShowListener {
                             override fun onUnityAdsShowFailure(
                                 placementId: String?,
                                 error: UnityAds.UnityAdsShowError?,
                                 message: String?
                             ) {
                                 handler.postDelayed({
                                     when (unityAdId) {
                                         UNITY_INTERESTIAL_AD_ID -> {
                                             loadInterestialAds(result, pos + 1)
                                         }
                                         UNITY_REWARD_AD_ID -> {
                                             loadRewardAds(result, pos + 1)
                                         }
                                     }
                                 }, 10L)
                             }

                             override fun onUnityAdsShowStart(placementId: String?) {

                             }

                             override fun onUnityAdsShowClick(placementId: String?) {

                             }

                             override fun onUnityAdsShowComplete(
                                 placementId: String?,
                                 state: UnityAds.UnityAdsShowCompletionState?
                             ) {
                                 interestialAdInterval = Calendar.getInstance().timeInMillis
                                 channel?.invokeMethod(METHOD_SUCCESS, null)
                             }
                         }
                     }

                     if (mUnityAdsLoadCallback == null) {
                         mUnityAdsLoadCallback = object : IUnityAdsLoadListener {
                             override fun onUnityAdsAdLoaded(placementId: String?) {
                                 when (unityAdId) {

                                     UNITY_INTERESTIAL_AD_ID -> {
                                         mInterstitialAdIsLoading = false
                                     }

                                     UNITY_REWARD_AD_ID -> {
                                         mRewardAdIsLoading = false
                                     }

                                 }
                                 handler.postDelayed({
                                     result?.let {

                                         progressBar?.dismiss()
                                         showUnityAds(result, unityAdId)
                                     }
                                 }, 10L)
                             }

                             override fun onUnityAdsFailedToLoad(
                                 placementId: String?,
                                 error: UnityAds.UnityAdsLoadError?,
                                 message: String?
                             ) {

                                 handler.postDelayed({
                                     when (unityAdId) {
                                         UNITY_INTERESTIAL_AD_ID -> {
                                             mInterstitialAdIsLoading = false
                                             loadInterestialAds(result, pos + 1)
                                         }
                                         UNITY_REWARD_AD_ID -> {
                                             mRewardAdIsLoading = false
                                             loadRewardAds(result, pos + 1)
                                         }
                                     }
                                 }, 10L)
                             }

                         }
                     }

                     UnityAds.load(unityAdId, mUnityAdsLoadCallback)

                 } else {
                     when (unityAdId) {
                         UNITY_INTERESTIAL_AD_ID -> {
                             loadInterestialAds(result, pos + 1)
                         }
                         UNITY_REWARD_AD_ID -> {
                             loadRewardAds(result, pos + 1)
                         }
                     }
                 }

             } ?: kotlin.run {
                 when (unityAdId) {
                     UNITY_INTERESTIAL_AD_ID -> {
                         loadInterestialAds(result, pos + 1)
                     }
                     UNITY_REWARD_AD_ID -> {
                         loadRewardAds(result, pos + 1)
                     }
                 }
             }
         }
     }
 */

    private fun showInterestialAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
/*
        geAdMobInterstitialAd?.let {
            Log.e(TAG, "loadGoogleAdsInterestialAds:ADNW_DEBUG_LOGGER  initMobSdk is Set" )
            val interstitialState = AppHarbr.getInterstitialState(it)
            if (interstitialState == AdStateResult.BLOCKED) {
                mInterstitialAdIsLoading = false
                mGoogleInterstitialAd = null
                geAdMobInterstitialAd = null
                Log.e("showInterestialAds: ", "initMobSdk  --- AD Is Blocked")
//                channel?.invokeMethod(METHOD_ERROR, null)
                loadInterestialAds(act, result, pos)
                return@showInterestialAds
            } else {
                showLog("Google Ads Show Completed")
                try {
                    it.adMobInterstitialAd?.show(act)
                    return@showInterestialAds
                } catch (e: Exception) {
                    e.printStackTrace()
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return@showInterestialAds
                }
            }
        }*/

        mGoogleInterstitialAd?.let {
            showLog("Google Ads Show Completed")
            try {
                it.show(act)
            } catch (e: Exception) {
                e.printStackTrace()
                methodChannel?.invokeMethod(METHOD_ERROR, null)
            }
        } ?: kotlin.run {
            showLog("Error is  Google Ads Not Loading")
            methodChannel?.invokeMethod(METHOD_ERROR, null)
        }
    }

    private fun showFacebookInterestialAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
        mFacebookInterstitialAd?.let {
            showLog("Facebook Ads Show Completed")
            // sendSuccess(result, "Google Ads Show Completed")
            try {

                if (mFacebookInterstitialAd == null || mFacebookInterstitialAd?.isAdLoaded == false) {
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return@showFacebookInterestialAds
                }

                if (mFacebookInterstitialAd?.isAdInvalidated == true) {
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return@showFacebookInterestialAds
                }

                it.show()
            } catch (e: Exception) {
                methodChannel?.invokeMethod(METHOD_ERROR, null)
            }
        } ?: kotlin.run {
            showLog("Error is  Facebook Ads Not Loading")
            methodChannel?.invokeMethod(METHOD_ERROR, null)
        }
    }

    private fun showAppLovinInterestialAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
        mInterstitialAppLovinAd?.let {
            showLog("Applovin Ads Show Completed")
            // sendSuccess(result, "Ads Show Completed")
            if (it.isReady) {
                it.showAd()
            } else {
                showLog("Error is Applovin Ads Not Loading")
                /*sendSuccess(
                    result,
                    false
                )*/
                methodChannel?.invokeMethod(METHOD_ERROR, null)
                // sendError(result, "APPLOVIN01", "  Error is Applovin Ads Loading But Not Ready")
            }
        } ?: kotlin.run {
            showLog("Error is Applovin Ads Not Loading")
            /*sendSuccess(
                result,
                false
            )*/
            methodChannel?.invokeMethod(METHOD_ERROR, null)
            // sendError(result, "1003", "  Error is Applovin Ads Not Loading at showing ad")
        }
    }

    /*private fun showUnityAds(result: MethodChannel.Result?, unityAdId: String) {
        try {
            // sendSuccess(result, "UNITY SUCCESS")
            UnityAds.show(
                this@MainActivity,
                unityAdId,
                UnityAdsShowOptions(),
                mUnityAdsShowCallback
            )
        } catch (e: Exception) {
            e.printStackTrace()
            // sendError(result, "UNITY01", e.toString())
            *//*sendSuccess(
                result,
                false
            )*//*
            channel?.invokeMethod(METHOD_ERROR, null)
        }
    }
*/

    private fun loadCustomInterestialAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
        /*val intent = Intent(
            this,
            CustomInterstitialAdsPopup::class.java
        )
        intent.flags =
            Intent.FLAG_ACTIVITY_REORDER_TO_FRONT or Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_NO_ANIMATION
        startActivity(intent)*/

        settingResponse?.let {
            it.customAds?.let { it ->
                if (it.isInterstitialAds == true) {
                    // sendSuccess(result, "SUCCESS")
                    mInterstitialAdIsLoading = false

                    progressBar?.dismiss()
                    if (it.qurakaIntraImage.size > 0) {
                        CustomAdDialog(act, it, object : CustomAdDialog.DialogInterface {
                            override fun onDismiss() {
                                // sendSuccess(result, true)
                                methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                            }

                            override fun onFailed() {
                                // sendSuccess(result, false)
                                methodChannel?.invokeMethod(METHOD_ERROR, null)
                            }
                        }).show()
                    } else {
                        loadInterestialAds(act, result, pos + 1)
                    }
                } else {
                    loadInterestialAds(act, result, pos + 1)
                }
            } ?: kotlin.run {
                loadInterestialAds(act, result, pos + 1)
            }
        }
    }

    private fun loadBannerAd(act: Activity, result: MethodChannel.Result?, call_id: String?) {
        settingResponse?.let {
            if (it.googleAds?.isBannerAds == true ||
                it.appLovin?.isBannerAds == true ||
                it.customAds?.isBannerAds == true ||
                it.unityAds?.isBannerAds == true
            ) {
                if (messenger != null) {
                    if (call_id != null) {
                        BannerAdControllerManager.createController(
                            appLovinSDK,
                            call_id,
                            messenger,
                            act
                        )
                    }
                }
            }
        }
    }

    private fun loadNativeAd(act: Activity, result: MethodChannel.Result?, call_id: String?) {
        settingResponse?.let {
            if (it.googleAds?.isNativeAds == true ||
                it.appLovin?.isNativeAds == true
            ) {
                if (call_id != null) {
                    if (messenger != null) {
                        NativeAdControllerManager.createController(
                            appLovinSDK,
                            call_id,
                            messenger,
                            act
                        )
                    }
                }
            }
        }
    }

    private fun destroyBannerAd(act: Activity, result: MethodChannel.Result, call_id: String?) {
        val id = call_id ?: return result.success(false)
        BannerAdControllerManager.getController(id)?.adMobBannerView?.destroy()
        try {
            BannerAdControllerManager.getController(id)?.bannerAdView?.removeAllViews()
        } catch (e: Exception) {
        }
        try {
            BannerAdControllerManager.removeController(id)
        } catch (e: Exception) {
        }
        sendSuccess(result, "DESTROY")
    }

    private fun destroyNativeAd(act: Activity, result: MethodChannel.Result, call_id: String?) {
        val id = call_id ?: return result.success(false)
        NativeAdControllerManager.removeController(id)
        sendSuccess(result, "DESTROY")
    }

    fun requestGoogleOpenAd(act: Activity, result: MethodChannel.Result? = null, pos: Int = 0) {
        settingResponse?.let {
            showLog("Open Ad is Already Loading Progress...  $pos  ${it.adsSequence.size}")
            if (it.adsSequence.size > 0 && it.adsSequence.size > pos
            ) {
                showLog("Open Ad is Already Loading Progress...  ${it.adsSequence[pos].lowercase()}")
                when (it.adsSequence[pos].lowercase()) {
                    act.getString(R.string.google_ads).lowercase() -> {
                        it.googleAds?.let {
                            showLog("Open Ad Google")
                            it.appOpenId?.let { appOpenId ->
                                showLog("Open Ad Google  $appOpenId")
                                if (it.isOpenAds == true) {
                                    showLog("Open Ad Google  ${it.isOpenAds}")
                                    if (mOpenAdIsLoading) {
                                        showLog("Open Ad is Already Loading Progress...")
                                        return
                                    }

                                    if (mOpenAd != null) {
                                        showLog("Open Ad is Already Loading Completed")
                                        handler.postDelayed({
                                            progressBar?.dismiss()
                                            showOpenAdIfAvailable(act, result, pos)
                                        }, 10L)
                                        return
                                    }

                                    mOpenAdIsLoading = true


                                    // progressBar?.show()

                                    if (mOpenAdLoadCallback == null) {
                                        mOpenAdLoadCallback =
                                            object : AppOpenAd.AppOpenAdLoadCallback() {
                                                override fun onAdLoaded(ad: AppOpenAd) {
                                                    mOpenAdIsLoading = false
                                                    mOpenAd = ad
                                                    handler.postDelayed({
                                                        progressBar?.dismiss()
                                                        showOpenAdIfAvailable(act, result, pos)
                                                    }, 10L)
                                                }

                                                override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                                                    mOpenAdIsLoading = false

                                                    progressBar?.dismiss()
                                                    showLog(
                                                        " -------- onAdFailedToLoad OpenAd  --- $loadAdError"
                                                    )
                                                    requestGoogleOpenAd(act, result, pos + 1)
                                                }
                                            }
                                    }
                                    showLog("Open Ad Google Request Send")
                                    mOpenAdLoadCallback?.let {
                                        AppOpenAd.load(
                                            act,
                                            appOpenId,
                                            getGoogleAdRequest(),
                                            AppOpenAd.APP_OPEN_AD_ORIENTATION_PORTRAIT,
                                            it
                                        )
                                        return@requestGoogleOpenAd
                                    }
                                }
                            }
                        }
                    }
                    act.getString(R.string.custom_ads).lowercase() -> {
                        it.customAds?.let { customAds ->
                            if (customAds.isOpenAds == true) {

                                if (mOpenAdIsLoading) {
                                    showLog("Open Ad is Already Loading Progress...")
                                    return
                                }

                                mOpenAdIsLoading = true

                                CustomOpenAdDialog(
                                    act,
                                    customAds,
                                    it.appTitle,
                                    it.appLogo,
                                    object : CustomOpenAdDialog.DialogInterface {
                                        override fun onDismiss() {
                                            mOpenAdIsLoading = false
                                            methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                                        }

                                        override fun onFailed() {
                                            mOpenAdIsLoading = false
                                            methodChannel?.invokeMethod(METHOD_ERROR, null)
                                        }
                                    }).show()

                                return@requestGoogleOpenAd
                            }
                        }
                    }
                    else -> {}
                }
            } else {
                progressBar?.dismiss()
                return@requestGoogleOpenAd
            }
        }
        requestGoogleOpenAd(act, result, pos + 1)
    }

    private fun showOpenAdIfAvailable(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        // If the app open ad is already showing, do not show the ad again.
        if (mOpenAdIsShowing) {
            Log.e("Open AD: ", "isShowingAd return")
            return
        }

        // If the app open ad is not available yet, invoke the callback then load the ad.
        if (mOpenAd == null) {
            progressBar?.dismiss()
            Log.e("Open AD: ", "isAd not Available() return")
            return
        }

        if (mOpenAdShowCallback == null) {
            mOpenAdShowCallback = object : FullScreenContentCallback() {
                /** Called when full screen content is dismissed.  */
                override fun onAdDismissedFullScreenContent() {
                    // Set the reference to null so isAdAvailable() returns false.

                    showLog("Open AD: :  onAdDismissedFullScreenContent   Open Ad Request")
                    mOpenAd = null
                    mOpenAdIsShowing = false
                    progressBar?.dismiss()
                    if (result != null) {
                        sendSuccess(result, "SUCCESS")
                    }
                }

                override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                    super.onAdFailedToShowFullScreenContent(adError)
                    showLog(
                        "ADOpen onAdFailedToShowFullScreenContent"
                    )
                    Log.e("Open AD: ", "onAdFailedToShowFullScreenContent   Open Ad Request")
                    mOpenAd = null
                    mOpenAdIsShowing = false
                    progressBar?.dismiss()
                    requestGoogleOpenAd(act, result, pos + 1)
                }

                /** Called when fullscreen content is shown.  */
                override fun onAdShowedFullScreenContent() {
                    progressBar?.dismiss()
                    showLog(
                        "open_ad ADOpen onAdShowedFullScreenContent"
                    )
                }
            }
        }
        mOpenAd?.fullScreenContentCallback = mOpenAdShowCallback
        mOpenAdIsShowing = true
        mOpenAd?.show(act)
    }

    private fun getGoogleAdRequest(): AdRequest {
        return AdRequest.Builder().build()
    }

    private fun showLog(error: String) {
        Log.e(TAG, error)
    }

    private fun sendSuccess(result: MethodChannel.Result?, msg: String) {
        try {
            handler.postDelayed({
                methodChannel?.invokeMethod(msg, null)
                //  result?.success(msg)
            }, 10L)
        } catch (e: Exception) {
        }
    }

    private fun loadRewardAds(act: Activity, result: MethodChannel.Result? = null, pos: Int = 0) {
        settingResponse?.let {
            if (it.adsSequence.size > 0 && it.adsSequence.size > pos) {
                isRewardVideoShowCompleted = false
                when (it.adsSequence[pos].lowercase()) {
                    act.getString(R.string.google_ads).lowercase() -> {
                        loadGoogleAdsRewardAds(act, result, pos)
                    }
                    act.getString(R.string.facebook_ads).lowercase() -> {
                        loadFacebookAdsRewardAds(act, result, pos)
                    }
                    act.getString(R.string.app_lovin).lowercase() -> {
                        loadAppLovinRewardAds(act, result, pos)
                    }
                    else -> {
                        loadRewardAds(act, result, pos + 1)
                    }
                }
            } else {
                mRewardAdIsLoading = false

                progressBar?.dismiss()
                methodChannel?.invokeMethod(METHOD_ERROR, null)
                /*sendSuccess(result, false)*/
            }
        }
    }

    private fun loadGoogleAdsRewardAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        settingResponse?.let { setting ->
            setting.googleAds?.let {
                if (it.isRewardedAds == true) {

                    if (it.rewardedId.isNullOrEmpty()) {
                        loadRewardAds(act, result, pos + 1)
                        return
                    }

                    if (mRewardAdIsLoading) {
                        showLog("Reward Ad is Already Loading Progress...")
                        //methodChannel?.invokeMethod(METHOD_ERROR, null)
                        return
                    }

                    if (mGoogleRewardedAd != null) {
                        showLog("Reward Ad is Already Loading Completed")
                        // methodChannel?.invokeMethod(METHOD_ERROR, null)
                        showGoogleRewardedAds(act, result, pos)
                        return
                    }

                    mRewardAdIsLoading = true


                    progressBar?.show()

                    if (mRewardedAdShowCallback == null) {
                        mRewardedAdShowCallback = object : FullScreenContentCallback() {
                            override fun onAdClicked() {
                                showLog("   --------  Reward onAdClicked ---   ")
                                super.onAdClicked()
                            }

                            override fun onAdDismissedFullScreenContent() {
                                super.onAdDismissedFullScreenContent()
                                mGoogleRewardedAd = null
//                                geAdMobRewardAd = null
                                progressBar?.dismiss()
                                showLog("   -------- onAdDismissedFullScreenContent Reward Show  $mGoogleInterstitialAd")
                                // sendSuccess(result, true)
                                if (isRewardVideoShowCompleted) {
                                    isRewardVideoShowCompleted = false
                                    methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                                }
//
                            }

                            override fun onAdShowedFullScreenContent() {
                                super.onAdShowedFullScreenContent()

                                progressBar?.dismiss()
                                showLog(" -------- onAdShowedFullScreenContent Reward Show  --- ")
                            }

                            override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                                super.onAdFailedToShowFullScreenContent(adError)

                                progressBar?.dismiss()
                                showLog(" -------- onAdFailedToShowFullScreenContent Reward Show  --- $adError")
                                /*result?.error(
                                    adError.code.toString(),
                                    "   -------- onAdShowedFullScreenContent Reward Show  ---   ${adError.message}",
                                    null
                                )*/
                                // sendSuccess(result, false)
                                handler.postDelayed({
                                    loadRewardAds(act, result, pos + 1)
                                }, 10L)
                            }

                            override fun onAdImpression() {
                                super.onAdImpression()
                                showLog(" -------- onAdImpression Reward Show  --- ")
                            }
                        }
                    }

                    if (mRewardedAdLoadCallback == null) {
                        mRewardedAdLoadCallback =
                            object : RewardedAdLoadCallback() {
                                override fun onAdLoaded(@NonNull rewardedAd: RewardedAd) {
                                    super.onAdLoaded(rewardedAd)
                                    mRewardAdIsLoading = false
                                    mGoogleRewardedAd = rewardedAd
                                    handler.postDelayed({
                                        result?.let {

                                            progressBar?.dismiss()
                                            showGoogleRewardedAds(act, result, pos)
                                        }
                                    }, 10L)
                                    mGoogleRewardedAd?.fullScreenContentCallback =
                                        mRewardedAdShowCallback
                                }

                                override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                                    super.onAdFailedToLoad(loadAdError)
                                    mRewardAdIsLoading = false
                                    mGoogleRewardedAd = null
//                                    geAdMobRewardAd = null
                                    showLog(
                                        " -------- onAdFailedToLoad Rewarded  --- $loadAdError"
                                    )
                                    handler.postDelayed({
                                        loadRewardAds(act, result, pos + 1)
                                    }, 10L)
                                }
                            }
                    }

                    mRewardedAdLoadCallback?.let { callback ->
                        ahRewardWrapperListener?.let { callback ->
                            RewardedAd.load(
                                act, it.rewardedId!!, getGoogleAdRequest(),
                                callback
                            )
                        } ?: kotlin.run {
                            RewardedAd.load(
                                act, it.rewardedId!!, getGoogleAdRequest(),
                                callback
                            )
                        }
                    }
                } else {
                    loadRewardAds(act, result, pos + 1)
                }
            } ?: kotlin.run {
                loadRewardAds(act, result, pos + 1)
            }
        }
    }

    private fun loadFacebookAdsRewardAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        settingResponse?.let {
            it.facebookAds?.let {
                if (it.isRewardedAds == true) {

                    if (it.rewardedId.isNullOrEmpty()) {
                        loadRewardAds(act, result, pos + 1)
                        return
                    }

                    if (mRewardAdIsLoading) {
                        showLog("Reward Ad is Already Loading Progress...")
                        // methodChannel?.invokeMethod(METHOD_ERROR, null)
                        return
                    }

                    if (mFacebookRewardedAd != null) {
                        showLog("Reward Ad is Already Loading Completed")
                        // methodChannel?.invokeMethod(METHOD_ERROR, null)
                        return
                    }

                    mRewardAdIsLoading = true


                    progressBar?.show()

                    if (mFacebookRewardedAdLoadCallback == null) {
                        mFacebookRewardedAdLoadCallback =
                            object : RewardedVideoAdListener {
                                override fun onError(p0: Ad?, p1: FBAdError?) {
                                    mFacebookRewardedAd = null
                                    mRewardAdIsLoading = false
                                    showLog(
                                        " -------- onAdFailedToLoad Rewarded  --- ${p1?.errorMessage}"
                                    )
                                    handler.postDelayed({
                                        loadRewardAds(act, result, pos + 1)
                                    }, 10L)
                                }

                                override fun onAdLoaded(p0: Ad?) {
                                    mRewardAdIsLoading = false
                                    handler.postDelayed({
                                        result?.let {
                                            progressBar?.dismiss()
                                            showFacebokRewardedAds(act, result, pos)
                                        }
                                    }, 10L)
                                    mGoogleRewardedAd?.fullScreenContentCallback =
                                        mRewardedAdShowCallback
                                }

                                override fun onAdClicked(p0: Ad?) {}

                                override fun onLoggingImpression(p0: Ad?) {}

                                override fun onRewardedVideoCompleted() {}

                                override fun onRewardedVideoClosed() {
                                    progressBar?.dismiss()
                                    mRewardAdIsLoading = false
                                    mFacebookRewardedAd = null
                                    methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                                }
                            }
                    }

                    mFacebookRewardedAdLoadCallback?.let { mFacebookRewardedAdLoadCallback ->
                        mFacebookRewardedAd = RewardedVideoAd(act, it.rewardedId)
                        mFacebookRewardedAd?.loadAd(
                            mFacebookRewardedAd?.buildLoadAdConfig()
                                ?.withAdListener(mFacebookRewardedAdLoadCallback)?.build()
                        )
                    }
                } else {
                    loadRewardAds(act, result, pos + 1)
                }
            } ?: kotlin.run {
                loadRewardAds(act, result, pos + 1)
            }
        }
    }

    private fun loadAppLovinRewardAds(
        act: Activity,
        result: MethodChannel.Result? = null,
        pos: Int
    ) {
        appLovinSDK?.let { appLovinSDK ->
            settingResponse?.let {
                it.appLovin?.let { it ->
                    if (it.isRewardedAds == true) {
                        it.appLovinRewardedID?.let { rewardedId ->

                            if (rewardedId.isNullOrEmpty()) {
                                loadRewardAds(act, result, pos + 1)
                                return@loadAppLovinRewardAds
                            }

                            if (mRewardAdIsLoading) {
                                progressBar?.dismiss()
                                showLog("AppLovin Reward Ad is Already Loading Progress...")
                                // methodChannel?.invokeMethod(METHOD_ERROR, null)
                                return@loadAppLovinRewardAds
                            }

                            if (mRewardedAppLovinAd != null) {
                                progressBar?.dismiss()
                                showLog("Applovin Reward Ad is Already Loading Completed")
                                // methodChannel?.invokeMethod(METHOD_ERROR, null)
                                return@loadAppLovinRewardAds
                            }

                            mRewardAdIsLoading = true
                            showLog("AppLovin Reward Ad is Started ...")

                            progressBar?.show()

                            if (mRewardedAppLovinCallback == null) {
                                mRewardedAppLovinCallback = object : MaxRewardedAdListener {

                                    override fun onAdLoaded(maxAd: MaxAd?) {
                                        // Reset retry attempt
//                                  retryAttempt = 0
                                        mRewardAdIsLoading = false
                                        handler.postDelayed({
                                            result?.let {
                                                progressBar?.dismiss()
                                                showAppLovinRewardedAds(act, result, pos)
                                            }
                                        }, 10L)
                                    }

                                    override fun onAdLoadFailed(
                                        adUnitId: String?,
                                        error: MaxError?
                                    ) {
                                        showLog("onAdLoadFailed: AppLovin Ad Failed  ------ ")
                                        mRewardedAppLovinAd = null
                                        mRewardAdIsLoading = false

                                        progressBar?.dismiss()
                                        handler.postDelayed({
                                            loadRewardAds(act, result, pos + 1)
                                        }, 10L)
                                        //                    retryAttempt++
                                        /*val delayMillis: Long = TimeUnit.SECONDS.toMillis(
                                            Math.pow(
                                                2.0,
                                                Math.min(6, retryAttempt).toDouble()
                                            ).toLong()
                                        )
                                        Handler(Looper.getMainLooper()).postDelayed({ interstitialAd?.loadAd() }, delayMillis)*/
                                    }

                                    override fun onAdDisplayFailed(
                                        maxAd: MaxAd?,
                                        error: MaxError?
                                    ) {
                                        // Interstitial ad failed to display. AppLovin recommends that you load the next ad.
                                        // mInterstitialAppLovinAd?.loadAd()
                                        mRewardAdIsLoading = false

                                        mRewardedAppLovinAd = null
                                        progressBar?.dismiss()
                                        showLog("onAdDisplayFailed: AppLovin Ad Failed   ------    ")
                                        handler.postDelayed({
                                            loadRewardAds(act, result, pos + 1)
                                        }, 10L)
                                    }

                                    override fun onRewardedVideoStarted(ad: MaxAd?) {

                                    }

                                    override fun onRewardedVideoCompleted(ad: MaxAd?) {
                                    }

                                    override fun onUserRewarded(ad: MaxAd?, reward: MaxReward?) {

                                    }

                                    override fun onAdDisplayed(maxAd: MaxAd?) {

                                        progressBar?.dismiss()
                                    }

                                    override fun onAdClicked(maxAd: MaxAd?) {

                                    }

                                    override fun onAdHidden(maxAd: MaxAd?) {
                                        // Interstitial ad is hidden. Pre-load the next ad
                                        progressBar?.dismiss()

                                        mRewardAdIsLoading = false
                                        mRewardedAppLovinAd = null
                                        methodChannel?.invokeMethod(METHOD_SUCCESS, null)
                                    }

                                }
                            }

                            mRewardedAppLovinAd =
                                MaxRewardedAd.getInstance(rewardedId, appLovinSDK, act)
                            mRewardedAppLovinAd?.setListener(mRewardedAppLovinCallback)
                            mRewardedAppLovinAd?.loadAd()

                        }
                    } else {
                        loadRewardAds(act, result, pos + 1)
                    }
                } ?: kotlin.run {
                    loadRewardAds(act, result, pos + 1)
                }
            } ?: kotlin.run {
                loadRewardAds(act, result, pos + 1)
            }
        } ?: kotlin.run {
            loadRewardAds(act, result, pos + 1)
        }
    }

    private fun showGoogleRewardedAds(act: Activity, result: MethodChannel.Result?, pos: Int) {

        /*geAdMobRewardAd?.let {
            val rewardedState = AppHarbr.getRewardedState(it)
            if (rewardedState == AdStateResult.BLOCKED) {
                Log.e("showGoogleRewardedAds: ", " Ad Is Blocked")
                // methodChannel?.invokeMethod(METHOD_ERROR, null)
                mRewardAdIsLoading = false
                mGoogleRewardedAd = null
                geAdMobRewardAd = null
                loadRewardAds(act, result, pos)
                return@showGoogleRewardedAds
            } else {
                showLog("Google Reward Ads Show Completed")
                try {
                    it.adMobRewardedAd?.show(act, OnUserEarnedRewardListener { rewardItem: RewardItem? ->
                        showLog("pragnesh palsana")
                        mGoogleRewardedAd = null
                        isRewardVideoShowCompleted = true
                    })
                    return@showGoogleRewardedAds
                } catch (e: Exception) {
                    e.printStackTrace()
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return@showGoogleRewardedAds
                }
            }
        }*/

        mGoogleRewardedAd?.let {
            showLog("Google Reward Ads Show Completed")
            // sendSuccess(result, "Google Reward Ads Show Completed")
            try {
                it.show(act, OnUserEarnedRewardListener { rewardItem: RewardItem? ->
                    showLog("pragnesh palsana")
                    mGoogleRewardedAd = null
                    isRewardVideoShowCompleted = true
                })
            } catch (e: Exception) {
                e.printStackTrace()
                methodChannel?.invokeMethod(METHOD_ERROR, null)
            }
        } ?: kotlin.run {
            showLog("Error is  Google Ads Not Loading")
            methodChannel?.invokeMethod(METHOD_ERROR, null)
        }
    }

    private fun showFacebokRewardedAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
        mFacebookRewardedAd?.let {
            showLog("Facebook Reward Ads Show Completed")
            try {

                if (mFacebookRewardedAd == null || mFacebookRewardedAd?.isAdLoaded == false) {
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return
                }

                if (mFacebookRewardedAd?.isAdInvalidated == true) {
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                    return
                }

                mFacebookRewardedAd?.show()

            } catch (e: Exception) {
                e.printStackTrace()
                methodChannel?.invokeMethod(METHOD_ERROR, null)
            }
        } ?: kotlin.run {
            showLog("Error is  Google Ads Not Loading")
            methodChannel?.invokeMethod(METHOD_ERROR, null)
        }
    }

    fun showAppLovinRewardedAds(act: Activity, result: MethodChannel.Result?, pos: Int) {
        mRewardedAppLovinAd?.let {
            try {
                if (it.isReady()) {
                    showLog("Applovin Reward Ads Show Completed")
                    it.showAd()
                } else {
                    showLog("Applovin Reward Ads Show Not Ready")
                    methodChannel?.invokeMethod(METHOD_ERROR, null)
                }
            } catch (e: Exception) {
                e.printStackTrace()
                methodChannel?.invokeMethod(METHOD_ERROR, null)
            }
        } ?: kotlin.run {
            showLog("Error is  Applovin Reward Ads Not Loading")
            methodChannel?.invokeMethod(METHOD_ERROR, null)
        }
    }

/*    // GeoSDK Apply
    @SuppressLint("MissingPermission")
    fun loadGeoSDK(act: Activity, GEOEDGE_SDK_KEY: String, listener: OnAppHarbrInitializationCompleteListener) {
        // progressBar?.show()
        val ahSdkConfiguration = AHSdkConfiguration.Builder(GEOEDGE_SDK_KEY)
            .withAdNetworksToMonitor(
                arrayOf(
                    AdSdk.ADCOLONY,
                    AdSdk.ADFALCON,
                    AdSdk.ADMOB,
                    AdSdk.ADMOST,
                    AdSdk.AEQUUS,
                    AdSdk.APPLOVIN,
                    AdSdk.CHARTBOOST,
                    AdSdk.CROSS_PROMOTION,
                    AdSdk.FACEBOOK,
                    AdSdk.FLURRY,
                    AdSdk.FYBER,
                    AdSdk.GAM,
                    AdSdk.HYPRMX,
                    AdSdk.IMOBILE,
                    AdSdk.INMOBI,
                    AdSdk.IRONSOURCE,
                    AdSdk.LEADBOLT,
                    AdSdk.MAIO,
                    AdSdk.MAX,
                    AdSdk.MESON,
                    AdSdk.MINTEGRAL,
                    AdSdk.MOBFOX,
                    AdSdk.MYTARGET,
                    AdSdk.NEND,
                    AdSdk.PANGLE,
                    AdSdk.SMAATO,
                    AdSdk.SCALEMONK,
                    AdSdk.SNAP,
                    AdSdk.TAPDAQ,
                    AdSdk.TAPJOY,
                    AdSdk.TENCENT,
                    AdSdk.UNITY,
                    AdSdk.VERIZON,
                    AdSdk.VPON,
                    AdSdk.VUNGLE,
                    AdSdk.ZUCKS
                )
            ).build()

        AppHarbr.initialize(
            act,
            ahSdkConfiguration,
            listener
        )
    }*/

}
