package com.btcpiyush.ads.google_applovin_unity_ads.banner_ad

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.content.res.Resources
import android.graphics.drawable.Drawable
import android.net.Uri
import android.util.DisplayMetrics
import android.util.Log
import android.view.Gravity
import android.view.View
import android.widget.FrameLayout
import android.widget.LinearLayout
import android.widget.RelativeLayout
import androidx.appcompat.widget.AppCompatImageView
import com.applovin.mediation.MaxAd
import com.applovin.mediation.MaxAdViewAdListener
import com.applovin.mediation.MaxError
import com.applovin.mediation.ads.MaxAdView
import com.bumptech.glide.load.DataSource
import com.bumptech.glide.load.engine.GlideException
import com.bumptech.glide.request.target.Target
import com.facebook.ads.Ad
import com.google.android.gms.ads.*
import com.btcpiyush.ads.google_applovin_unity_ads.RequestFactory
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.GoogleAdsSetting
import com.btcpiyush.ads.google_applovin_unity_ads.image_loader.ImageLoadGlide
import com.btcpiyush.ads.google_applovin_unity_ads.R as R;
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import java.util.*
import com.facebook.ads.AdError as FacebookAdError
import com.facebook.ads.AdListener as FacebookAdListener
import com.facebook.ads.AdSize as FacebookAdSize
import com.facebook.ads.AdView as FacebookAdView


class BannerAdViewFactory(val activity: Activity, val settings: AppSettings) :
    PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    init {
        Log.e("MainActivity.TAG", "BannerAdViewFactory Init")
    }

    override fun create(context: Context?, viewId: Int, args: Any?): PlatformView {
        val creationParams = args as Map<String?, Any?>?
        Log.e("create: ", " ---- " + creationParams.toString())
        return loadBanner(activity, context!!, settings, creationParams)
    }
}

private fun loadBanner(
    activity: Activity,
    context: Context,
    it: AppSettings,
    creationParams: Map<String?, Any?>?,
    pos: Int = 0,
    extrapos: Int = 0
): PlatformView {
    if (it.adsSequence.size > 0 && it.adsSequence.size > pos) {
        when (it.adsSequence[pos].lowercase()) {
            context.getString(R.string.google_ads).lowercase() -> {
                val googleBannerID = if (it.googleAds?.isBannerAds == true) {
                    it.googleAds?.bannerId
                } else {
                    null
                }
                googleBannerID?.let { banner_id ->
                    return AbmobBannerAdView(activity, context, it, banner_id, creationParams, pos)
                } ?: run {
                    return loadBanner(activity, context, it, creationParams, pos + 1)
                }
            }
            context.getString(R.string.extra_google_ads).lowercase() -> {
                it.extraGoogleAds?.let { extraGoogleAds ->
                    if (extraGoogleAds.size > 0 && extraGoogleAds.size > extrapos) {
                        extraGoogleAds[extrapos]?.let { googleAds ->
                            val googleBannerID = if (googleAds.isBannerAds == true) {
                                googleAds.bannerId
                            } else {
                                null
                            }
                            googleBannerID?.let { banner_id ->
                                return AbmobBannerAdView(activity, context, it, banner_id, creationParams, pos, extrapos, extraGoogleAds)
                            } ?: run {
                                return loadBanner(activity, context, it, creationParams, pos, extrapos + 1)
                            }
                        } ?: run {
                            return loadBanner(activity, context, it, creationParams, pos, extrapos + 1)
                        }
                    } else {
                        return loadBanner(activity, context, it, creationParams, pos + 1)
                    }
                } ?: run {
                    return loadBanner(activity, context, it, creationParams, pos + 1)
                }
            }
            context.getString(R.string.facebook_ads).lowercase() -> {
                val facebookBannerID = if (it.facebookAds?.isBannerAds == true) {
                    it.facebookAds?.bannerId
                } else {
                    null
                }
                facebookBannerID?.let { banner_id ->
                    return FacebookBannerAdView(
                        activity,
                        context,
                        it,
                        banner_id,
                        creationParams,
                        pos
                    )
                } ?: run {
                    return loadBanner(activity, context, it, creationParams, pos + 1)
                }
            }
            context.getString(R.string.app_lovin).lowercase() -> {
                val appLovinBannerID = if (it.appLovin?.isBannerAds == true) {
                    it.appLovin?.appLovinBannerID
                } else {
                    null
                }
                appLovinBannerID?.let { banner_id ->
                    return AppLovinBannerAdView(
                        activity,
                        context,
                        it,
                        banner_id,
                        creationParams,
                        pos
                    )
                } ?: run {
                    return loadBanner(activity, context, it, creationParams, pos + 1)
                }
            }
            /*context.getString(R.string.unity_ads).lowercase() -> {
                val unityGameID = if (it.unityAds?.isBannerAds == true) {
                    it.unityAds?.unityId
                } else {
                    null
                }
                unityGameID?.let { gameID ->
                    return UnityBannerAdView(activity, context, it, gameID, creationParams, pos)
                } ?: run {
                    return loadBanner(activity, context, it, creationParams, pos + 1)
                }
            }*/
            context.getString(R.string.custom_ads).lowercase() -> {
                return CustomAdView(activity, context, it, creationParams, pos)
            }
            else -> {
                return DefaultClass(context)
            }
        }
    } else {
        return DefaultClass(context)
        /*mInterstitialAdIsLoading = false
        progressBar?.dismiss()*/
    }
}

data class AbmobBannerAdView(
    val activity: Activity,
    val context: Context,
    val it: AppSettings,
    val bannerId: String,
    val data: Map<String?, Any?>?,
    val pos: Int,
    val extrapos: Int = -1,
    val extraList:ArrayList<GoogleAdsSetting?>? = null
) : PlatformView {

    private var controller: BannerAdController =
        BannerAdControllerManager.getController(data!!["controllerId"] as String)!!
    private var adSize: AdSize
    private var nonPersonalizedAds: Boolean
    private var keywords: List<String>

    private fun getAdSize(context: Context, width: Float): AdSize {
        return AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(context, width.toInt())
    }

    init {
        adSize = getAdSize(controller.context, (data!!["size_width"] as Double).toFloat())
        nonPersonalizedAds = data["nonPersonalizedAds"] as Boolean
        keywords = data["keywords"] as List<String>
        generateAdView(context, bannerId, data)
        controller.loadRequested = { load(it) }
        load(null)
    }

    @SuppressLint("MissingPermission")
    private fun load(result: MethodChannel.Result?) {
        controller.adMobBannerView?.adListener = object : AdListener() {
            override fun onAdImpression() {
                super.onAdImpression()
                /* Log.e( "onAdImpression: ", " =========")
                 controller.channel.invokeMethod("onAdImpression", null)*/
            }

            override fun onAdClicked() {
                super.onAdClicked()
                /*Log.e( "onAdClicked: ", " =========")
                controller.channel.invokeMethod("onAdClicked", null)*/
            }

            override fun onAdFailedToLoad(error: LoadAdError) {
                super.onAdFailedToLoad(error)
                Log.e("onAdFailedToLoad: ", " =========")
                /*controller.channel.invokeMethod("onAdFailedToLoad", encodeError(error))*/
                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViews()
                    }
                } catch (e: Exception) {
                }

                try {
                    controller.bannerAdView?.addView(
                        loadBanner(
                            activity, context,
                            it, data, pos + 1
                        ).view as FrameLayout
                    )
                } catch (e: Exception) { }
            }

            override fun onAdLoaded() {
                super.onAdLoaded()
                Log.e("onAdLoaded: ", " =========")
                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViewsInLayout()
                    }
                } catch (e: Exception) {
                }
                // controller?.bannerAdView = null
                try {
                    controller.bannerAdView?.addView(controller.adMobBannerView)
                } catch (e: Exception) {
                }
                controller.channel.invokeMethod("onAdLoaded",controller.adMobBannerView?.adSize?.height ?: 0)
                result?.success(true)
                /*  controller.channel.invokeMethod("onAdLoaded", controller.adMobBannerView?.adSize?.height ?: 0)
                  result?.success(true)*/
            }
        }

/*        controller.adMobBannerView?.let{ adView ->
            if(it.adSetting?.isGeoSDKFlag == true && (it.adSetting?.geoSDKKey ?: "").isNotEmpty()){
                AppHarbr.addBannerView(
                    AdSdk.ADMOB,
                    adView
                ) { _: Any?, _: String?, _: AdFormat?, _: Array<AdBlockReason?>? ->
                    Log.d(
                        "LOG",
                        "AppHarbr - On Banner Blocked"
                    )
                }
            }
        }*/

        controller.adMobBannerView?.loadAd(
            RequestFactory.createAdRequest(
                nonPersonalizedAds,
                keywords
            )
        )
    }

    private fun generateAdView(context: Context, bannerId: String, data: Map<String?, Any?>?) {
        if (controller.bannerAdView == null) {
            controller.bannerAdView = FrameLayout(context)
        }
        controller.adMobBannerView = AdView(context)
        data?.let { data ->
            val displaymetrics = DisplayMetrics()
            activity.windowManager.defaultDisplay.getMetrics(displaymetrics)
            val h = displaymetrics.heightPixels
            val width: Int = (data["size_width"] as Double).toInt()
            val height: Int = (data["size_height"] as Double).toInt()
            if (height != -1) controller.adMobBannerView?.setAdSize(AdSize(width, height))
            else controller.adMobBannerView?.setAdSize(adSize)

            /*controller?.bannerAdView?.layoutParams = FrameLayout.LayoutParams(displaymetrics.widthPixels, if (height != -1) {
                height
            } else {
                adSize.height
            })*/

            val param = FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT,
                FrameLayout.LayoutParams.WRAP_CONTENT
            )
            param.gravity = Gravity.CENTER
            controller.bannerAdView?.layoutParams = param
            controller.adMobBannerView?.adUnitId = bannerId
            Log.e(
                "generateAdView: ",
                " ========= $bannerId   ${controller.bannerAdView?.layoutParams?.width}      ${controller.bannerAdView?.layoutParams?.height}"
            )
        }
    }

    override fun getView(): View {
        return controller.bannerAdView!!
    }

    override fun dispose() {
        try {
            controller.adMobBannerView?.destroy()
            if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                controller.bannerAdView?.removeAllViews();
            }
        } catch (e: Exception) {
        }
    }

    fun encodeError(error: AdError?): Map<String, Any?> {
        return mapOf<String, Any?>(
            "errorCode" to error?.code,
            "domain" to error?.domain,
            "message" to error?.message
        )
    }
}

data class AppLovinBannerAdView(
    val activity: Activity,
    val context: Context,
    val it: AppSettings,
    val bannerId: String,
    val data: Map<String?, Any?>?,
    val pos: Int
) : PlatformView {

    private var controller: BannerAdController =
        BannerAdControllerManager.getController(data!!["controllerId"] as String)!!
    private var adSize: AdSize
    private var appLovinBannerAdView: MaxAdView? = null

    private fun getAdSize(context: Context, width: Float): AdSize {
        return AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(context, width.toInt())
    }

    init {
        adSize = getAdSize(controller.context, (data!!["size_width"] as Double).toFloat())
        if (controller.bannerAdView == null) {
            controller.bannerAdView = FrameLayout(activity)
        }
        generateAdView(context, bannerId, data)
        controller.loadRequested = { load(it) }
        load(null)
    }

    private fun load(result: MethodChannel.Result?) {
        appLovinBannerAdView?.setListener(object : MaxAdViewAdListener {
            override fun onAdLoaded(ad: MaxAd?) {
                // bannerRetryAttempt = 0
                Log.e("onAdLoaded: ", " =========  AppLovin")
                // controller.channel.invokeMethod("loaded", null)
                controller.channel.invokeMethod("onAdLoaded", dpToPx(50))
                result?.success(true)
            }

            override fun onAdDisplayed(ad: MaxAd?) {
                Log.e("onAdDisplayed: ", " =========  AppLovin")
                //controller.channel.invokeMethod("onAdLoaded", appLovinBannerAdView?.placement ?: "0")
                //result?.success(true)
            }

            override fun onAdHidden(ad: MaxAd?) {
                Log.e("onAdHidden: ", " =========  AppLovin")
                controller.channel.invokeMethod("onAdHidden", null)
            }

            override fun onAdClicked(ad: MaxAd?) {
                Log.e("onAdClicked: ", " =========  AppLovin")
                controller.channel.invokeMethod("onAdClicked", null)
            }

            override fun onAdLoadFailed(adUnitId: String?, error: MaxError?) {
                Log.e("onAdLoadFailed: ", " =========  AppLovin")
                //controller.channel.invokeMethod("onAdFailedToLoad", encodeError(error))
                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViews();
                    }
                } catch (e: Exception) {
                }
//                controller?.bannerAdView = null
                try {
                    controller.bannerAdView?.addView(
                        loadBanner(
                            activity, context,
                            it, data, pos + 1
                        ).view as FrameLayout
                    )
                } catch (e: Exception) {
                }
                // result?.success(false)
                // Rewarded ad failed to load
                // We recommend retrying with exponentially higher delays up to a maximum delay (in this case 64 seconds)

                // Rewarded ad failed to load
                // We recommend retrying with exponentially higher delays up to a maximum delay (in this case 64 seconds)
                /* bannerRetryAttempt++
                val delayMillis = TimeUnit.SECONDS.toMillis(
                    Math.pow(2.0, Math.min(6, bannerRetryAttempt).toDouble())
                        .toLong()
                )

                Handler(Looper.getMainLooper()).postDelayed({ bannerAdView?.loadAd() }, delayMillis)*/
            }

            override fun onAdDisplayFailed(ad: MaxAd?, error: MaxError?) {
                Log.e("onAdDisplayFailed: ", " =========  AppLovin ======  ${error.toString()}")
                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViews();
                    }
                } catch (e: Exception) {
                }
                // controller?.bannerAdView = null
                try {
                    controller.bannerAdView?.addView(
                        loadBanner(
                            activity, context,
                            it, data, pos + 1
                        ).view as FrameLayout
                    )
                } catch (e: Exception) {
                }
//                result?.success(false)
                /*controller.channel.invokeMethod("onAdFailedToLoad", encodeError(error))
                result?.success(false)*/
            }

            override fun onAdExpanded(ad: MaxAd?) {
                Log.e("onAdExpanded: ", " =========  AppLovin")
            }

            override fun onAdCollapsed(ad: MaxAd?) {
                Log.e("onAdCollapsed: ", " =========  AppLovin")
            }

        })
        controller.bannerAdView?.addView(appLovinBannerAdView)
        appLovinBannerAdView?.loadAd()
        appLovinBannerAdView?.startAutoRefresh()
    }

    private fun generateAdView(context: Context, bannerId: String, data: Map<String?, Any?>?) {
        appLovinBannerAdView = MaxAdView(bannerId, controller.appLovinSDK, context)
        data?.let { data ->
            val width: Int = RelativeLayout.LayoutParams.MATCH_PARENT
            val height = dpToPx(50)

            appLovinBannerAdView?.layoutParams = FrameLayout.LayoutParams(width, height)
            val param = FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT,
                FrameLayout.LayoutParams.WRAP_CONTENT
            )
            param.gravity = Gravity.CENTER
            controller.bannerAdView?.layoutParams = param
        }
        Log.e(
            "generateAdView: ",
            " ========= $bannerId   ${controller.bannerAdView?.layoutParams?.width}      ${controller.bannerAdView?.layoutParams?.height}"
        )
    }

    override fun getView(): View {
        return controller.bannerAdView!!
    }

    override fun dispose() {
        try {
            appLovinBannerAdView?.destroy()
            if (controller.bannerAdView?.childCount ?: 0 > 0) {
                controller.bannerAdView?.removeAllViews();
            }
        } catch (e: Exception) {
        }
    }

    fun dpToPx(dp: Int): Int {
        return (dp * Resources.getSystem().displayMetrics.density).toInt()
    }

    fun encodeError(error: MaxError?): Map<String, Any?> {
        return mapOf<String, Any?>(
            "errorCode" to error?.code,
            "domain" to error?.waterfall?.name,
            "message" to error?.message
        )
    }
}

data class FacebookBannerAdView(
    val activity: Activity,
    val context: Context,
    val it: AppSettings,
    val bannerId: String,
    val data: Map<String?, Any?>?,
    val pos: Int
) : PlatformView {

    private var controller: BannerAdController =
        BannerAdControllerManager.getController(data!!["controllerId"] as String)!!
    private var adSize: AdSize

    private fun getAdSize(context: Context, width: Float): AdSize {
        return AdSize.getCurrentOrientationAnchoredAdaptiveBannerAdSize(context, width.toInt())
    }

    init {
        adSize = getAdSize(controller.context, (data!!["size_width"] as Double).toFloat())
        generateAdView(context, bannerId, data)
        controller.loadRequested = { load(it) }
        load(null)
    }

    private fun load(result: MethodChannel.Result?) {
        val adListener: FacebookAdListener = object : FacebookAdListener {

            override fun onError(p0: Ad?, p1: FacebookAdError?) {
                Log.e("onAdFailedToLoad: ", " =========  ${p1?.errorMessage}")
                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViews();
                    } else {
                        controller.bannerAdView?.removeAllViews()
                    }
                } catch (e: Exception) {
                }
                // controller?.bannerAdView = null
                try {
                    controller.bannerAdView?.addView(
                        loadBanner(
                            activity, context,
                            it, data, pos + 1
                        ).view as FrameLayout
                    )
                } catch (e: Exception) {
                }
            }

            override fun onAdLoaded(ad: Ad?) {

                try {
                    if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                        controller.bannerAdView?.removeAllViewsInLayout()
                    }
                } catch (e: Exception) {
                }

                try {
                    controller.bannerAdView?.addView(controller.facebookBannerView)
                } catch (e: Exception) {
                }
                controller.channel.invokeMethod("onAdLoaded", FacebookAdSize.BANNER_HEIGHT_50.height)
                result?.success(true)
            }

            override fun onAdClicked(ad: Ad?) {

            }

            override fun onLoggingImpression(ad: Ad?) {

            }
        }
        controller.facebookBannerView?.loadAd(
            controller.facebookBannerView?.buildLoadAdConfig()?.withAdListener(adListener)?.build()
        )
    }

    private fun generateAdView(context: Context, bannerId: String, data: Map<String?, Any?>?) {
        if (controller.bannerAdView == null) {
            controller.bannerAdView = FrameLayout(activity)
        }
        controller.facebookBannerView =
            FacebookAdView(context, bannerId, FacebookAdSize.BANNER_HEIGHT_50)
        data?.let { data ->
            val param = FrameLayout.LayoutParams(
                FrameLayout.LayoutParams.MATCH_PARENT,
                FrameLayout.LayoutParams.WRAP_CONTENT
            )
            param.gravity = Gravity.CENTER
            controller.bannerAdView?.layoutParams = param
            Log.e(
                "generateAdView: ",
                " ========= $bannerId   ${controller.bannerAdView?.layoutParams?.width}      ${controller.bannerAdView?.layoutParams?.height}"
            )
        }
    }

    override fun getView(): View {
        return controller.bannerAdView!!
    }

    override fun dispose() {
        try {
            controller.adMobBannerView?.destroy()
            if (controller.bannerAdView?.childCount ?: 0 > 0) {
                controller.bannerAdView?.removeAllViews();
            }
        } catch (e: Exception) {
        }
    }

    fun encodeError(error: AdError?): Map<String, Any?> {
        return mapOf<String, Any?>(
            "errorCode" to error?.code,
            "domain" to error?.domain,
            "message" to error?.message
        )
    }
}

/*data class UnityBannerAdView(
    val activity: Activity,
    val context: Context,
    val it: AppSettings,
    val gameId: String,
    val data: Map<String?, Any?>?,
    val pos: Int
) : PlatformView {

    private var controller: BannerAdController =
        BannerAdControllerManager.getController(data!!["controllerId"] as String)!!

    init {
        generateAdView()
        controller.loadRequested = { load(it) }
        load(null)
    }

    private fun load(result: MethodChannel.Result?) {
        UnityBanners.setBannerPosition(BannerPosition.BOTTOM_CENTER)
        UnityBanners.setBannerListener(object : IUnityBannerListener {
            override fun onUnityBannerLoaded(placementId: String?, view: View?) {
                controller.bannerAdView?.let {
                    if (it.childCount > 0) {
                        it.removeAllViews()
                    }
                    it.addView(view)
                    controller.channel.invokeMethod("loaded", null)
                }

                Log.e("onUnityBannerLoaded: ", " =========  Unity")
                data?.let { data ->
                    val height: Int =
                        if ((data["size_height"] as Double).toInt() != -1) (data["size_height"] as Double).toInt()
                        else 50
                    controller.channel.invokeMethod("onAdLoaded", height)
                    result?.success(true)
                }
            }

            override fun onUnityBannerUnloaded(placementId: String?) {
                Log.e("onUnityBannerError: ", " =========  Unity")
//                controller.channel.invokeMethod("onAdFailedToLoad", "Unity")
                if (controller?.bannerAdView?.childCount ?: 0 > 0) {
                    controller?.bannerAdView?.removeAllViews();
                }
                controller?.bannerAdView = loadBanner(
                    activity, context,
                    it, data, pos + 1
                ).view as FrameLayout
//                result?.success(false)
            }

            override fun onUnityBannerShow(placementId: String?) {
                Log.e("onUnityBannerShow: ", " =========  Unity")
                *//*controller.channel.invokeMethod("onAdLoaded", "Unity")
                result?.success(true)*//*
            }

            override fun onUnityBannerClick(placementId: String?) {
                Log.e("onUnityBannerClick: ", " =========  Unity")
//                controller.channel.invokeMethod("onAdClicked", "Unity")
            }

            override fun onUnityBannerHide(placementId: String?) {
                Log.e("onUnityBannerHide: ", " =========  Unity")
//                controller.channel.invokeMethod("onAdHidden", "Unity")
            }

            override fun onUnityBannerError(message: String?) {
                Log.e("onUnityBannerError: ", " =========  Unity")
//                controller.channel.invokeMethod("onAdFailedToLoad", "Unity")
                if (controller?.bannerAdView?.childCount ?: 0 > 0) {
                    controller?.bannerAdView?.removeAllViews();
                }
                controller?.bannerAdView = loadBanner(
                    activity, context,
                    it, data, pos + 1
                ).view as FrameLayout
//                result?.success(false)
            }
        })
        UnityAds.initialize(activity, gameId, true)
        UnityBanners.loadBanner(activity, "banner")
    }

    private fun generateAdView() {
        if (controller.bannerAdView == null) {
            controller.bannerAdView = FrameLayout(activity)
        }
        data?.let { data ->
            val width: Int = (data["size_width"] as Double).toInt()
            val height: Int =
                if ((data["size_height"] as Double).toInt() != -1) (data["size_height"] as Double).toInt()
                else 50

            val param = FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, height)
            param.gravity = Gravity.CENTER_HORIZONTAL
            controller.bannerAdView?.layoutParams = param

            *//*controller.bannerAdView?.layoutParams = FrameLayout.LayoutParams(width, height)
            controller?.bannerAdView?.layoutParams = FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT)*//*
        }
    }

    override fun getView(): View {
        return controller.bannerAdView!!
    }

    override fun dispose() {
        try {
            UnityBanners.destroy()
            controller.bannerAdView?.let {
                if (it.childCount > 0) {
                    it.removeAllViews()
                }
            }
        } catch (e: Exception) {
        }
    }

}*/

data class CustomAdView(
    val activity: Activity,
    val context: Context,
    val it: AppSettings,
    val data: Map<String?, Any?>?,
    val pos: Int
) : PlatformView {

    private var controller: BannerAdController =
        BannerAdControllerManager.getController(data!!["controllerId"] as String)!!

    init {
        generateAdView()
        controller.loadRequested = { load(it) }
        load(null)
    }

    private fun load(result: MethodChannel.Result?) {
        val imageView = AppCompatImageView(activity)
        // setting height and width of imageview
        imageView.layoutParams =
            LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, dpToPx(50))
        //Toast.makeText(this,new Random().nextInt(Custom_Ads_Model.ads_List.size()) +" : custom Ads : "+ Custom_Ads_Model.ads_List.size(),Toast.LENGTH_SHORT).show();
        val customAds = it.customAds
        val bannersList = customAds?.qurakaBannerImage
        val totalBanners = bannersList?.size ?: 0
        if (totalBanners > 0) {
            var random_index = Random().nextInt(totalBanners)

            if (random_index >= totalBanners) {
                random_index = totalBanners - 1
            }

            ImageLoadGlide().imageLoad(
                bannersList!![random_index],
                imageView,
                activity,
                object : ImageLoadGlide.ImageLoad {
                    override fun onLoadFailed(
                        e: GlideException?,
                        model: Any?,
                        target: Target<Drawable?>?,
                        isFirstResource: Boolean
                    ) {
                        // controller.channel.invokeMethod("onAdFailedToLoad", encodeError(error))
                        try {
                            if ((controller.bannerAdView?.childCount ?: 0) > 0) {
                                controller.bannerAdView?.removeAllViews();
                            }
                        } catch (e: Exception) {
                        }
//                controller?.bannerAdView = null
                        try {
                            controller.bannerAdView?.addView(
                                loadBanner(
                                    activity, context,
                                    it, data, pos + 1
                                ).view as FrameLayout
                            )
                        } catch (e: Exception) {
                        }
                        // result?.success(false)
                    }

                    override fun onSuccess(
                        resource: Drawable?,
                        model: Any?,
                        target: com.bumptech.glide.request.target.Target<Drawable?>?,
                        dataSource: DataSource?,
                        isFirstResource: Boolean
                    ) {
                        controller.channel.invokeMethod("onAdLoaded", dpToPx(50))
                        result?.success(true)
                    }

                }
            )

            imageView.setOnClickListener { _ ->
                customAds.qurekaLink?.let {
                    if (it.isNotEmpty()) {
                        try {
                            val browserIntent =
                                Intent(Intent.ACTION_VIEW, Uri.parse(it))
                            browserIntent.setPackage("com.android.chrome")
                            context.startActivity(browserIntent)
                        } catch (e: Exception) {
                            try {
                                val browserIntent =
                                    Intent(Intent.ACTION_VIEW, Uri.parse(it))
                                context.startActivity(browserIntent)
                            } catch (e: Exception) {
                            }
                        }
                    }
                }
            }

            controller.bannerAdView?.addView(imageView)
        } else {
            try {
                if (controller.bannerAdView?.childCount ?: 0 > 0) {
                    controller.bannerAdView?.removeAllViews();
                }
            } catch (e: Exception) {
            }
            try {
                controller.bannerAdView?.addView(
                    loadBanner(
                        activity, context,
                        it, data, pos + 1
                    ).view as FrameLayout
                )
            } catch (e: Exception) {
            }
        }

    }

    fun encodeError(error: MaxError?): Map<String, Any?> {
        return mapOf<String, Any?>(
            "errorCode" to error?.code,
            "domain" to error?.waterfall?.name,
            "message" to error?.message
        )
    }

    private fun generateAdView() {
        if (controller.bannerAdView == null) {
            controller.bannerAdView = FrameLayout(activity)
        }
        data?.let { data ->
            val height: Int = dpToPx(50)
            val param = FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, height)
            param.gravity = Gravity.CENTER
            controller.bannerAdView?.layoutParams = param
        }
    }

    fun dpToPx(dp: Int): Int {
        return (dp * Resources.getSystem().displayMetrics.density).toInt()
    }

    override fun getView(): View {
        return controller.bannerAdView!!
    }

    override fun dispose() {
        try {
            controller.bannerAdView?.let {
                if (it.childCount > 0) {
                    it.removeAllViews()
                }
            }
        } catch (e: Exception) {
        }
    }

}

data class DefaultClass(val context: Context) : PlatformView {


    override fun getView(): View {
        return FrameLayout(context)
    }

    override fun dispose() {}

}
