package com.btcpiyush.ads.google_applovin_unity_ads.interestial_ad

import android.app.Activity
import android.util.Log
import com.applovin.mediation.MaxAd
import com.applovin.mediation.MaxAdListener
import com.applovin.mediation.MaxError
import com.applovin.mediation.ads.MaxInterstitialAd
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin
import com.btcpiyush.ads.google_applovin_unity_ads.ProgressDialog
import com.btcpiyush.ads.google_applovin_unity_ads.R
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppLovinSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.AppSettings
import com.btcpiyush.ads.google_applovin_unity_ads.api.FacebookAdsSetting
import com.btcpiyush.ads.google_applovin_unity_ads.api.GoogleAdsSetting
import com.facebook.ads.Ad
import com.facebook.ads.InterstitialAdListener
import com.google.android.gms.ads.AdError
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.FullScreenContentCallback
import com.google.android.gms.ads.LoadAdError
import com.google.android.gms.ads.interstitial.InterstitialAd
import com.facebook.ads.InterstitialAd as FBInterstitialAd
import com.google.android.gms.ads.interstitial.InterstitialAdLoadCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.util.*

class AdmobInterstitial(
    private val flutterPluginBinding: FlutterPlugin.FlutterPluginBinding,
    private val act: Activity,
    private val settingResponse: AppSettings
) :
    MethodChannel.MethodCallHandler {

    companion object {
        val allAds: MutableMap<Int, InterestialModel> = mutableMapOf()
        val TAG: String = "ADMOBINTER ADS"
    }

    private fun getGoogleAdRequest(): AdRequest {
        return AdRequest.Builder().build()
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {

        when (call.method) {
            "call.ads.interestial.isLoaded" -> {
                val id = call.argument<Int>("id")
                val model = allAds[id] ?: return result.success(false)

                if (model.isLoaded)
                    result.success(true)
                else
                    result.success(false)
            }

            "call.ads.interestial" -> {
                val id: Int = call.argument<Int?>("id") ?: 0

                showLog("id  ======  $id")

                if (allAds[id] == null) {
                    allAds[id] = InterestialModel(
                        progressBar = ProgressDialog(
                            act,
                            settingResponse.adSetting?.adsLoadingText ?: "Ads will loading..."
                        )
                    )
                }

                val model = allAds[id]

                model?.let { model ->
                    settingResponse.adSetting?.interestialAdsIntervalCount?.let { count ->
                        settingResponse.adSetting?.interestialAdsIntervalCount = count + 1
                    }
                    model.pos = 0
                    loadIntretialAds(id, model)
                }
            }
            "call.ads.interestial.dispose" -> {
                val id = call.argument<Int>("id")
                allAds.remove(id)
            }
            else -> result.notImplemented()
        }
    }

    private fun loadIntretialAds(id: Int, model: InterestialModel) {

        val isInterestial = ((settingResponse.adSetting?.interestialAdsInterval
            ?: 0) > 0 && settingResponse.adSetting?.interestialAdsIntervalCount?.mod(
            settingResponse.adSetting?.interestialAdsInterval ?: 0
        ) == 0 && (GoogleApplovinUnityAdsPlugin.interestialAdInterval + ((settingResponse.adSetting?.interestialAdsTimingInterval
            ?: 0) * 1000) <= Calendar.getInstance().timeInMillis || GoogleApplovinUnityAdsPlugin.interestialAdInterval == 0L))

        val pos = model.pos++
        showLog("pos ======== $pos")
        if (settingResponse.adsSequence.size > 0 && settingResponse.adsSequence.size > pos) {
            when (settingResponse.adsSequence[pos].lowercase()) {
                act.getString(R.string.google_ads).lowercase() -> {
                    settingResponse.googleAds?.let { google_ads ->
                        if ((google_ads.interstitialInterval <= 0 && isInterestial) || (google_ads.interstitialInterval > 0 && settingResponse.adSetting?.interestialAdsIntervalCount?.mod(
                                google_ads.interstitialInterval
                            ) == 0)
                        ) {
                            loadGoogleInterestialAds(
                                id,
                                model,
                                google_ads
                            )
                            return@loadIntretialAds
                        }
                    }
                }

                act.getString(R.string.facebook_ads).lowercase() -> {
                    settingResponse.facebookAds?.let { facebookAds ->
                        if ((facebookAds.interstitialInterval <= 0 && isInterestial) || (facebookAds.interstitialInterval > 0 && settingResponse.adSetting?.interestialAdsIntervalCount?.mod(
                                facebookAds.interstitialInterval
                            ) == 0)
                        ) {
                            loadFacebookInterestialAds(
                                id,
                                model,
                                facebookAds
                            )
                            return@loadIntretialAds
                        }
                    }
                }

                act.getString(R.string.app_lovin).lowercase() -> {
                    settingResponse.appLovin?.let{ appLovinAds ->
                        if ((appLovinAds.isUnlimitedAds == true) || isInterestial) {
                            loadAppLovinInterestialAds(
                                id,
                                model,
                                appLovinAds
                            )
                            return@loadIntretialAds
                        }
                    }

                }
            }

            loadIntretialAds(id, model)
        } else {
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_INTERESTIAL_NAME}_$id"
            )
            model.isInProgress = false
            model.isLoaded = false
            model.progressBar.dismiss()
            adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_ERROR, null)
        }
    }

    private fun loadGoogleInterestialAds(id: Int,
                                         model: InterestialModel,
                                         it: GoogleAdsSetting) {
        if (model.isInProgress == true) {
            return@loadGoogleInterestialAds
        }

        if (model.interstialAd != null) {
            showInterestialAds(id, model)
            return@loadGoogleInterestialAds
        }

        val admobID = it.interstitialId ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_INTERESTIAL_NAME}_$id"
            )
            showLog("$admobID is not empty ")
            val callback = object : InterstitialAdLoadCallback() {
                override fun onAdLoaded(interstitialAd: InterstitialAd) {
                    super.onAdLoaded(interstitialAd)
                    model.isInProgress = false
                    model.isLoaded = true
                    model.interstialAd = interstitialAd
                    model.progressBar.dismiss()
                    showInterestialAds(id, model)
                    model.interstialAd?.fullScreenContentCallback =
                        object : FullScreenContentCallback() {
                            override fun onAdClicked() {
                                showLog("   --------  Interestial onAdClicked ---   ")
                                // sendSuccess(result, "   --------  Interestial onAdClicked ---   ")
                                super.onAdClicked()
                            }

                            override fun onAdDismissedFullScreenContent() {
                                super.onAdDismissedFullScreenContent()
                                model.interstialAd = null
                                model.progressBar.dismiss()
                                GoogleApplovinUnityAdsPlugin.interestialAdInterval =
                                    Calendar.getInstance().timeInMillis
                                showLog("   -------- onAdDismissedFullScreenContent Show ")

                                adChannel?.invokeMethod(
                                    GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS,
                                    null
                                )
                            }

                            override fun onAdShowedFullScreenContent() {
                                super.onAdShowedFullScreenContent()

                                model.progressBar.dismiss()
                                showLog(" -------- onAdShowedFullScreenContent Show  --- ")
                            }

                            override fun onAdFailedToShowFullScreenContent(adError: AdError) {
                                super.onAdFailedToShowFullScreenContent(adError)
                                showLog(
                                    " -------- onAdFailedToShowFullScreenContent Show  --- $adError"
                                )
                            }

                            override fun onAdImpression() {
                                super.onAdImpression()
                                showLog(
                                    " -------- onAdImpression Show  --- "
                                )
                            }
                        }
                }

                override fun onAdFailedToLoad(loadAdError: LoadAdError) {
                    super.onAdFailedToLoad(loadAdError)
                    model.isInProgress = false
                    model.isLoaded = false
                    model.interstialAd = null
                    loadIntretialAds(id, model)
                }
            }

            InterstitialAd.load(act, admobID, getGoogleAdRequest(), callback)
            return@loadGoogleInterestialAds
        }

        loadIntretialAds(id, model)
    }

    private fun loadFacebookInterestialAds(id: Int,
                                           model: InterestialModel,
                                           it: FacebookAdsSetting) {
        if (model.isInProgress == true) {
            showLog("Interestial Ad is Already Loading Progress...")
            return
        }

        if (model.fbInterstitialAd != null) {
            showLog("Interestial Ad is Already Loading Completed")
            if (model.isLoaded) {
                showFacebookInterestialAds(id, model)
            }
            return
        }

        val admobID = it.interstitialId ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_INTERESTIAL_NAME}_$id"
            )
            showLog("$admobID is not empty ")
            val callback = object : InterstitialAdListener {
                override fun onError(p0: Ad?, p1: com.facebook.ads.AdError?) {
                    showLog(
                        " -------- onAdFailedToShowFullScreenContent Show  --- $p1"
                    )
                    model.fbInterstitialAd = null
                    model.isInProgress = false
                    loadIntretialAds(id, model)
                }

                override fun onAdLoaded(p0: Ad?) {
                    model.isInProgress = false
                    model.isLoaded = true
                    showFacebookInterestialAds(id, model)
                }

                override fun onAdClicked(p0: Ad?) {}

                override fun onLoggingImpression(p0: Ad?) {}

                override fun onInterstitialDisplayed(p0: Ad?) {
                    model.progressBar.dismiss()
                    GoogleApplovinUnityAdsPlugin.interestialAdInterval = Calendar.getInstance().timeInMillis
                }

                override fun onInterstitialDismissed(p0: Ad?) {
                    model.fbInterstitialAd = null
                    model.isInProgress = false
                    model.progressBar.dismiss()
                    GoogleApplovinUnityAdsPlugin.interestialAdInterval = Calendar.getInstance().timeInMillis
                    showLog("   -------- onAdDismissedFullScreenContent Show ")
                    adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                }
            }

            model.fbInterstitialAd = FBInterstitialAd(act, admobID)
            model.fbInterstitialAd?.loadAd(
                model.fbInterstitialAd?.buildLoadAdConfig()
                    ?.withAdListener(callback)?.build()
            )
            return@loadFacebookInterestialAds
        }

        loadIntretialAds(id, model)

    }

    private fun loadAppLovinInterestialAds(id: Int,
                                           model: InterestialModel,
                                           it: AppLovinSetting) {
        if (model.isInProgress == true) {
            showLog("Interestial Ad is Already Loading Progress...")
            return
        }

        if (model.appLovinInterstitialAd != null) {
            showLog("Interestial Ad is Already Loading Completed")
            if (model.isLoaded) {
                showAppLovinInterestialAds(id, model)
            }
            return
        }

        val admobID = it.appLovinInterestialID ?: ""

        if (!admobID.isNullOrEmpty()) {
            model.isInProgress = true
            model.isLoaded = false
            model.progressBar.show()
            val adChannel = MethodChannel(
                flutterPluginBinding.binaryMessenger,
                "${GoogleApplovinUnityAdsPlugin.CHANNEL_INTERESTIAL_NAME}_$id"
            )
            showLog("$admobID is not empty ")
            val callback = object : MaxAdListener {

                override fun onAdLoaded(maxAd: MaxAd?) {
                    showLog(
                        " -------- onAdLoaded apploving 12312  --- "
                    )
                    model.isInProgress = false
                    model.isLoaded = true
                    model.progressBar.dismiss()
                    showAppLovinInterestialAds(id, model)
                }

                override fun onAdLoadFailed(adUnitId: String?, error: MaxError?) {
                    showLog("onAdLoadFailed: AppLovin Ad Failed  ------ ")
                    model.appLovinInterstitialAd = null
                    model.isInProgress = false
                    model.isLoaded = false
                    model.progressBar.dismiss()
                    loadIntretialAds(id, model)
                }

                override fun onAdDisplayFailed(maxAd: MaxAd?, error: MaxError?) {
                    // Interstitial ad failed to display. AppLovin recommends that you load the next ad.
                    // mInterstitialAppLovinAd?.loadAd()
                    model.appLovinInterstitialAd = null
                    model.isInProgress = false
                    showLog("onAdDisplayFailed: AppLovin Ad Failed   ------    ")
                    loadIntretialAds(id, model)
                }

                override fun onAdDisplayed(maxAd: MaxAd?) {
                    showLog(
                        " -------- onAdDisplayed apploving 12312  --- "
                    )
                }

                override fun onAdClicked(maxAd: MaxAd?) {

                }

                override fun onAdHidden(maxAd: MaxAd?) {
                    // Interstitial ad is hidden. Pre-load the next ad
                    model.isInProgress = false
                    model.appLovinInterstitialAd = null
                    GoogleApplovinUnityAdsPlugin.interestialAdInterval = Calendar.getInstance().timeInMillis
                    adChannel?.invokeMethod(GoogleApplovinUnityAdsPlugin.METHOD_SUCCESS, null)
                }

            }

            GoogleApplovinUnityAdsPlugin.appLovinSDK?.let { appLovinSDK ->
                model.appLovinInterstitialAd = MaxInterstitialAd(
                    it.appLovinInterestialID, appLovinSDK, act
                )

                model.appLovinInterstitialAd?.let{
                    it.setListener(callback)
                    it.loadAd()
                    return@loadAppLovinInterestialAds
                }
            }
        }

        loadIntretialAds(id, model)
    }

    private fun showInterestialAds(
        id: Int,
        model: InterestialModel
    ) {
        model.interstialAd?.let {
            try {
                it.show(act)
                return@showInterestialAds
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }
        loadIntretialAds(id,model)
    }

    private fun showFacebookInterestialAds(id: Int,
                                           model: InterestialModel) {
        model.fbInterstitialAd?.let {
            showLog("Facebook Ads Show Completed")
            try {
                if (it.isAdLoaded && !it.isAdInvalidated) {
                    it.show()
                    return@showFacebookInterestialAds
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }

        loadIntretialAds(id,model)
    }

    private fun showAppLovinInterestialAds(id: Int,
                                           model: InterestialModel) {
        model.appLovinInterstitialAd?.let {
            showLog("Applovin Ads Show Completed")
            if (it.isReady) {
                it.showAd()
                return@showAppLovinInterestialAds
            }
        }

        loadIntretialAds(id,model)

    }


    private fun showLog(error: String) {
        Log.e(TAG, error)
    }

}
